var AJAX = new Class({
	Implements: Options,
	loader: false,
	request: false,
	url: false,
	attempts: 0,
	
	start_time: 0,
	
	options: {
		response_format: 'txt',
		hide_response: false,
		onComplete: false,
		oncomplete_args: false,
		onError: false,
		allow_attempts: false, // if set to an integer, it will retry failed ajax calls X times
		error_response_check: false, // pass in a string to check for comparison of ajax response. if string exists inside response, it will do attempt
		silence_errors: false // if true, no errors will be thrown
	},
	
	is_completed: false,
	
	initialize: function(url, options) {
		// debug('AJAX.initialize');
		if (!network_monitor_obj.isAvailable()) {
			//error({ errorID: -6 });
			//return false;
		}
		
		this.url = url;
		this.setOptions(options);
		
		// debug('new air.URLRequest('+this.url+')');
		this.request = new air.URLRequest(this.url);
		this.start_time = (new Date()).getTime();
		
		var post_data = '';
		// debug('options.post ' + options.post)
		if (options.post) {
			$each(options.post, function(value, key) {
				post_data += '&' + key + '=' + escape(value);
			});
			post_data = post_data.substr(1);
			
			var vars = new air.URLVariables(post_data);
			this.request.method = air.URLRequestMethod.POST;
			//this.request.requestHeaders = [new air.URLRequestHeader('Content-Type', 'text/html; charset=utf-8')];
			this.request.data = vars;
		}
		
		// debug('hide_response ' + this.options.hide_response);
		if (this.options.hide_response) { 
			// debug('[AJAX: ' + this.url + ' post: ' + post_data + ']');
		}
		
		if( this.options.allow_attempts ) this.attempts = this.options.allow_attempts;
		
		// debug('new air.URLLoader()');
		this.loader = new air.URLLoader();
		this.loader.addEventListener(air.Event.COMPLETE, this.onComplete.bind(this));
		this.loader.addEventListener(air.IOErrorEvent.IO_ERROR, this.onIOError.bind(this));
		// this.loader.addEventListener(air.HTTPStatusEvent.HTTP_RESPONSE_STATUS, this.onHTTPResponseStatus.bind(this));
		try {
			// debug('this.loader.load(this.request)', this.request, '----------------');
			this.loader.load(this.request);
			// debug('this.loader', this.loader, '-----------------');
		} catch ( e ) {
			// debug('catch error', e, '-------------------');
			// we know this is probably a type error
			error( {
				errorID: -21,
				text: e + ", type=" + typeof( url ) + ", url=" + url				
			});			
		}
	},
	
	removeListeners: function() {
		this.loader.removeEventListener(air.Event.COMPLETE, this.onComplete.bind(this));
		this.loader.removeEventListener(air.IOErrorEvent.IO_ERROR, this.onIOError.bind(this));
		// this.loader.removeEventListener(air.HTTPStatusEvent.HTTP_RESPONSE_STATUS, this.onHTTPResponseStatus.bind(this));
	},
		
	/**
	 * Attempts to make an ajax call X times if enabled
	 */
	retry: function() {
		if (this.options.allow_attempts > 0) {
			debug('[Attempt #' + (this.attempts - this.options.allow_attempts + 1) + ' of ' + this.attempts + ' AJAX: ' + this.url + ']');
			
			this.options.allow_attempts--;
			this.loader.load(this.request);
			return true;
		}
		
		return false;
	},

	getDuration: function() {
		// debug('AJAX.getDuration');
		return Utility.toFixed(((new Date().getTime()) - this.start_time) / 3600, 2) + 's';
	},
	
	onIOError: function(e) {
		// debug('AJAX.onIOError', e, '----------------');
		if (!network_monitor_obj.isAvailable()) {
			error({ errorID: -6 });
			return false;
		}
		
		debug('[Error: ' + e + ']')
		
		// Retry the ajax call
		if (this.retry()) 
			return false;
					
		this.removeListeners();
		
		if (!this.options.silence_errors) {
			// Log the error IOerror
			error({
				errorID: -7,
				text: e + ',' + this.url
			});
		}
	},
	
	onHTTPResponseStatus: function(e) {
		debug('onHTTPResponseStatus', e, '--------------------');
		error({
			errorID: -7,
			text: e.toString()
		});
		
		this.removeListeners();
	},
	
	onComplete: function(e) {
		// debug('onComplete', e, '-------------');
		var data = this.loader.data;
		
		// If this is a custom error check and it fails then do retry
		if (this.options.error_response_check != '') {
			if (data.contains(this.options.error_response_check)) { // string matched
				if (this.retry()) // ok retrying
					return false;
				// else {
				// 	if(this.options.onError) this.options.onError.call(e);
				// }
			}
		}
		
		// Check for null data
		if (data === null || data === undefined || data == '') {
			// debug('null data');
			if (this.retry()) { // try again, if not throw error
				if (this.options.response_format == 'json') {
					error({
						errorID: -2,
						text: 'Null response: ' + this.url
					});
					return false;
				}
			}
			// else
			// {
			// 	if(this.options.onError) this.options.onError.call(e);
			// }
		}
		
		if (!this.options.hide_response) { 
			// debug('[AJAX(' + this.getDuration() + '): ' + this.url + ' \n' + data + ']');
		}
		
		// debug('removeListeners');
		this.removeListeners();

		var raw_data = data;
		// debug('raw data', raw_data, '-------------');
		
		try {
			if (this.options.response_format == 'json') {
				data = JSON.decode(data);
			}
			
			// debug('JSON decoded data', data, '----------------');
			
			if (this.options.onComplete) 
				this.options.onComplete(data, this.options.oncomplete_args);
		}
		catch(e) {
			if (!this.options.silence_errors) {
				error({
					errorID: -3,
					text: e + ',' + this.url + ',' + raw_data + ',' + this.getDuration()
				});
			}
		}
		
		this.is_completed = true;
	}
});

BatchAJAX = new Class({
	calls: [],
	listener: false,
	on_complete: false,
	on_failed: false,
	failed_calls: 0,
	
	initialize: function(on_complete, on_failed) {
		// debug('//////// BATCH START /////////');
		this.on_complete = on_complete;
		this.on_failed = on_failed;
	},
	
	add: function(url, options) {
		this.calls.push( [url, options] );
	},
	
	addAJAX: function(ajax_obj) {
		this.calls.push(ajax_obj);
		$clear(this.listener);
		this.listener = this.onComplete.periodical(200, this);
	},
	
	start: function() {
		for(var i = 0; i < this.calls.length; i++) {
			this.calls[i] = new AJAX(this.calls[i][0], this.calls[i][1]);
		}
		this.listener = this.onComplete.periodical(200, this);
	},
	
	onComplete: function() {
		var ajax_call;
		for(var i = 0; i < this.calls.length; i++) {
			ajax_call = this.calls[i];
			
			try {	
				// debug('check onComplete ' + ajax_call.is_completed);
				if( ajax_call )
				{
					if(ajax_call.options.allow_attempts <= 0) throw "failed";
					if(!ajax_call.is_completed) return false;
				}
			}
			catch(e) {
				debug('onComplete error:', e, ajax_call), '===================';
				this.onFailed(e);
			}
		}
		
		// debug('//////// BATCH FINISHED /////////');
		
		$clear(this.listener);
		this.calls = false;
		
		// debug("Batch finished. Check for failed calls: " + this.failed_calls);
		if( this.failed_calls > 0 )
		{
			if(this.on_failed ) this.on_failed();
			// else this.on_complete();
			
			return false;
		}
		else if (this.on_complete) 
			this.on_complete();
	},
	
	onFailed: function(e) {
		// debug('Call Failed');
		this.failed_calls++;
	}
});