var AJAX = new Class({
	Implements: Options,
	loader: false,
	request: false,
	url: false,
	
	start_time: 0,
	
	options: {
		response_format: 'txt',
		hide_response: false,
		onComplete: false,
		oncomplete_args: false,
		allow_attempts: false, // if set to an integer, it will retry failed ajax calls X times
		error_response_check: false, // pass in a string to check for comparison of ajax response. if string exists inside response, it will do attempt
		silence_errors: false // if true, no errors will be thrown
	},
	
	is_completed: false,
	
	getPostParamsFromQuery: function( url ) {
		if( url.indexOf( "?" ) != -1 ) {
			var splitURL = url.split( "?" );
			splitURL = splitURL[1].split("&");
			var retArray = new Array();
			for( var i = 0; i < splitURL.length; i++ ) {
				var nvp = splitURL[ i ].split("=");
				if( nvp.length == 2 ) retArray.push( {name: nvp[0],value:nvp[1]} );
			}
			return retArray;
		} else return new Array();
	},
	
	initialize: function(url, options) {
		this.url = url;
		this.setOptions(options);
		
		//this.request = new air.URLRequest(this.url);
		this.request = new window.runtime.com.vimeo.net.URLRequest( this.url );
		
		this.start_time = (new Date()).getTime();
		
		var post_data = new Array();
		if (options.post) {
			$each(options.post, function(value, key) {
				post_data.push( {name:key, value: value} );
			});
			
			// convert query string params to post params
			post_data = post_data.concat( this.getPostParamsFromQuery( this.url ) );
			
			// now get rid of query string params
			var newURL = this.url.split("?")[ 0 ];
			this.request = new window.runtime.com.vimeo.net.URLRequest( newURL );
			this.request.data = post_data;
			this.request.method = air.URLRequestMethod.POST;
			
		}
		
		if (this.options.hide_response) { 
			//debug('[AJAX: ' + this.url + ' post: ' + post_data + ']');
		}
		
		//this.loader = new air.URLLoader();
		this.loader = new window.runtime.com.vimeo.net.URLLoader();
		this.loader.addEventListener(air.Event.COMPLETE, this.onComplete.bind(this));
		//this.loader.addEventListener(air.IOErrorEvent.IO_ERROR, this.onIOError.bind(this));
		try {
			this.loader.load(this.request);
		} catch ( e ) {
			// we know this is probably a type error
			error( {
				errorID: -21,
				text: e + ", type=" + typeof( url ) + ", url=" + url				
			});			
		}
	},
	
	removeListeners: function() {
		this.loader.removeEventListener(air.Event.COMPLETE, this.onComplete.bind(this));
		this.loader.removeEventListener(air.IOErrorEvent.IO_ERROR, this.onIOError.bind(this));
	},
		
	/**
	 * Attempts to make an ajax call X times if enabled
	 */
	retry: function() {
		if (this.options.allow_attempts > 0) {
			debug('[Attempt #' + this.options.allow_attempts + ' AJAX: ' + this.url + ']');
			
			this.options.allow_attempts--;
			this.loader.load(this.request);
			return true;
		}
		
		return false;
	},

	getDuration: function() {
		return Utility.toFixed(((new Date().getTime()) - this.start_time) / 3600, 2) + 's';
	},
	
	onIOError: function(e) {
		debug('[Error: ' + e + ']')
		
		// Retry the ajax call
		if (this.retry()) 
			return false;
					
		this.removeListeners();
		
		if (!this.options.silence_errors) {
			// Log the error IOerror
			error({
				errorID: -7,
				text: e + ',' + this.url
			});
		}
	},
	
	onComplete: function(e) {
		var data = this.loader.data;
		
		// If this is a custom error check and it fails then do retry
		if (this.options.error_response_check != '') {
			if (data.contains(this.options.error_response_check)) { // string matched
				if (this.retry()) // ok retrying
					return false;
			}
		}
		
		// Check for null data
		if (data === null || data === undefined || data == '') {
			if (this.retry()) { // try again, if not throw error
				if (this.options.response_format == 'json') {
					error({
						errorID: -2,
						text: 'Null response: ' + this.url
					});
					return false;
				}
			}
		}
		
		if (!this.options.hide_response) { 
			// debug('[AJAX(' + this.getDuration() + '): ' + this.url + ' \n' + data + ']');
		}
		
		this.removeListeners();

		var raw_data = data;
		
		try {
			if (this.options.response_format == 'json') {
				data = JSON.decode(data);
			}
			
			if (this.options.onComplete) 
				this.options.onComplete(data, this.options.oncomplete_args);
		}
		catch(e) {
			if (!this.options.silence_errors) {
				error({
					errorID: -3,
					text: e + ',' + this.url + ',' + raw_data + ',' + this.getDuration()
				});
			}
		}
		
		this.is_completed = true;
	}
});

BatchAJAX = new Class({
	calls: [],
	listener: false,
	on_complete: false,
	
	initialize: function(on_complete) {
		// debug('//////// BATCH START /////////');
		this.on_complete = on_complete;
	},
	
	add: function(url, options) {
		this.calls.push( [url, options] );
	},
	
	addAJAX: function(ajax_obj) {
		this.calls.push(ajax_obj);
		$clear(this.listener);
		this.listener = this.onComplete.periodical(200, this);
	},
	
	start: function() {
		for(var i = 0; i < this.calls.length; i++) {
			this.calls[i] = new AJAX(this.calls[i][0], this.calls[i][1]);
		}
		this.listener = this.onComplete.periodical(200, this);
	},
	
	onComplete: function() {
		for(var i = 0; i < this.calls.length; i++) {
			if (!this.calls[i].is_completed) return false;
		}
		
		// debug('//////// BATCH FINISHED /////////');
		
		$clear(this.listener);
		this.calls = false;
		if (this.on_complete) 
			this.on_complete();
	}
});