/* 
	Written by Robert Nyman, http://www.robertnyman.com
	Modified by Casey Pugh, http://www.caseypugh.com
	Hacked up beyond recognition by Joseph Schmitt
	Inspired by http://www.insideria.com/2008/03/air-api-performing-updates-in-1.html, and then modified
*/	
var AIRUpdater = function() {
	// This is only used for the filename of the installer
	var applicationName = "VimeoUploader_0.9.5.2.air";
	var applicationVersion = 0;
	var latestVersion = 0;
	/*
		URL to go to to check the value of the <version> and <releasenotes>
		The value in version is compared to the one in the applications XML setup file,
		and an update dialog is triggered if the <version> in the below XML file is higher
		From there on, suggested alternatives for the end user is to 
		start an update or cancel it for the moment
		
		Suggested XML structure:
		
		<?xml version="1.0" encoding="UTF-8"?>
		<application>
			<latestversion>0.7</latestversion>
			<releasenotes>Added automatic update downloading feature.</releasenotes>
		</application>
	*/
	var latestVersionCheckUrl = "http://" + VIMEO_URL + '/' + 'assets/air/desktopuploader' + '/' + 'version.xml';
	var updateAvailable = null;
	var updateAvailableDialog = null;
	var releaseNotes = null;
	var releaseNotesText = "";
	var releaseRequired = null;
	/*
		Change this URL to the download URL of your AIR app. Version number 
		and ".air" extension will automatically be added; version taken 
		from the XML value found in the latestVersionCheckUrl page
	*/	
	var updaterUrl = "http://" + VIMEO_URL + '/' + 'assets/air/desktopuploader' +  '/' + 'VimeoUploader';
	var stream = null;
	var updateFile = null;
	
	var getApplicationVersion = function() {
		applicationVersion = Utility.getApplicationVersion();
	};
	
	var getLatestVersion = function() {
		/* 
			Checks for what the latest available version is 
			from the URL specified in latestVersionCheckUrl
		*/	
		var XMLHttp = new XMLHttpRequest();
		XMLHttp.onreadystatechange = function() {
			if (XMLHttp.readyState === 4) {
				var response = XMLHttp.responseXML;
				
				if (!response) return false;
				var releaseNotesNode = response.getElementsByTagName("releasenotes")[0];
				releaseRequired = response.getElementsByTagName("required")[0].firstChild.nodeValue == 1;
				
				/*
					Adds a reference to a releaseNote for the latest version, 
					IF a <releasenotes> node exists
				*/
				if (typeof releaseNotesNode === "object" && releaseNotesNode.firstChild) {
					releaseNotesText = releaseNotesNode.firstChild.wholeText;
				}
				var latestVersionNode = response.getElementsByTagName("latestversion")[0];
				/*
					Triggers a version comparison with the existing installed application, 
					IF a <latestversion> node exists
				*/	
				if (typeof latestVersionNode === "object" && latestVersionNode.firstChild) {
					latestVersion = latestVersionNode.firstChild.nodeValue;//parseFloat(latestVersionNode.firstChild.nodeValue, 10);
					compareVersions();
				}
			}
		};
		XMLHttp.open("GET", latestVersionCheckUrl + "?v=" + (Math.random() * 1000), true);
		XMLHttp.send(null);
	};
	
	var compareVersions = function() {
		var comparison = Utility.compareVersions(latestVersion, applicationVersion);
		if( comparison > 0 ) 
		{
			// Present release notes for the new version available
			if( releaseNotesText && releaseNotesText != '' )
			{
				$('autoupdate_notes').innerHTML = releaseNotesText;
				$('autoupdate_notes').show();
				$('autoupdate_lightbox').setWidth(500);
			}
			else
			{
				$('autoupdate_notes').hide();
				$('autoupdate_lightbox').setWidth('inherit');
			}
			
			// Add onclick event to start update button
			$('autoupdate_btn').addEvent('click', initUpdateApplication);
			
			if (releaseRequired) {
				$('autoupdate_required_txt').show();
				$('autoupdate_cancel_btn').hide();
			} else {
				$('autoupdate_required_txt').hide();
				$('autoupdate_cancel_btn').show();
			}
			
			autoupdate_lightbox.show();
			loading_lightbox.hide();
		}
	};
	
	var initUpdateApplication = function() {
		/* 
			The updating has started. Prefereably, you'd like to hide 
			or disable the start and cancel update buttons now
		*/
		stream = new air.URLStream();
		
		/*
			This event is recommend to give the end user continuous 
			feedback about how the update goes
		*/
		stream.addEventListener(air.ProgressEvent.PROGRESS, updatingStatus);
		stream.addEventListener(air.Event.COMPLETE, updateApplication);
		
		// debug('initUpdateApplication ' + updaterUrl + "_"+latestVersion+".air");
		// Note that the latest version number and ".air" extension is automatically added
		stream.load( new air.URLRequest(updaterUrl + "_"+latestVersion+".air"));
	};
	
	var updatingStatus = function (e) {
		// This is example code to show updating status
		var percentage = Math.round((e.bytesLoaded / e.bytesTotal) * 100);
		//document.getElementById("current-updating-status").innerHTML = percentage + "%";
		$('autoupdate_btn').disable('Updating ' + percentage + '% ...');
		$('autoupdate_cancel_btn').hide();
	};
	
	var updateApplication = function() {
		VimeoLogger.log('update', {
			version: latestVersion.toString()
		});
		
		var ba = new air.ByteArray();
		stream.readBytes(ba, 0, stream.bytesAvailable);
		updateFile = air.File.applicationStorageDirectory.resolvePath(applicationName);
		fileStream = new air.FileStream();
		fileStream.addEventListener( air.Event.CLOSE, installUpdate );
		fileStream.openAsync(updateFile, air.FileMode.WRITE);
		fileStream.writeBytes(ba, 0, ba.length);
	 	fileStream.close();
	};
	
	var installUpdate = function() {
		$('autoupdate_btn').disable('Installing ...');
		
		var updater = new air.Updater();
		// Notice that the version name has to be present as a second parameter
		try {
			updater.update(updateFile, latestVersion.toString());
		} catch(e) {
			$('autoupdate_btn').enable('Failed. Try again?');
		}
	};
	
	return {
		init: function() {
			if (!file_manager.is_uploading && !autoupdate_lightbox.is_visible && !is_user_idle) {
				applicationVersion = 0;
				latestVersion = 0;
				
				getApplicationVersion();
				getLatestVersion();
			}
		}
	};
}();
window.addEvent('load', air_updater_load);

function air_updater_load() 
{
	AIRUpdater.init.periodical(60000 * 5);
	AIRUpdater.init();
}