/**
 * File manager
 */
var FileManager = new Class({
	files: [],
	upload_ticket: false,
	loader: false,
	cur_file_pos: 0,
	sortable_list: false,
	is_uploading: false,
	is_paused: false,
	disk_usage: false,
	last_selected_file: false,
	upload_status: "",
	skinned_dropdown: false,
	lockControlsTimer: false,
	
	max_file_size: null,	
	total_allowable_uploads: 0,
	total_selectable_files: 0,
	controlsUnresponsiveDelay: 60000,
	
	prev_rate_data : null,
	
	system_locked: false, // if true, system is currently doing some work and freezing screen
	
	add_area: null,
	
	initialize: function()
	{ 
		// Load previous state of the uploader
		var f = this.getSessionFiles();
		
		for (var i = 0; i < f.length; i++) {
			var new_index = this.addFile(air.File.applicationStorageDirectory.resolvePath(f[i].path));
			
			if (new_index >= 0) {
				try {
					this.files[new_index].title 		= f[i].title;
					this.files[new_index].description 	= f[i].description;
					this.files[new_index].tags 			= f[i].tags;
					this.files[new_index].privacy 		= f[i].privacy;
					this.files[new_index].password 		= f[i].password;
					this.files[new_index].addto			= f[i].addto;
				} 
				catch (e) { error(e); }
			}
		}
		
		//Set up tabs
		$('metaverse').getElements('.right_column ul li h1').each(function(item, index, items){
			//Store the index inside the element
			$(item).store('index', index);
			
			//Click event
			$(item).addEvent('click', function(e){
				//if disabled, do nothing
				if( $(this).hasClass('disabled') || $(this).hasClass('active') ) return false;
				
				//Set all tabs to not active
				items.each(function(tab){
					$(tab).removeClass('active');
				});
				
				//Set clicked tab to active
				$(this).addClass('active');
				
				var contentAreas = $('metaverse').getElements('.right_column div.content > div');
				//Hide all of the content areas
				contentAreas.each(function(content){
					$(content).hide();
				});
				
				//Show selected content area
				contentAreas[$(this).retrieve('index')].show();
				
				var footers = $('metaverse').getElements('.right_column div.footers > div');
				//Hide all of the content areas
				footers.each(function(footer){
					$(footer).hide();
				});
				
				//Show selected content area
				footers[$(this).retrieve('index')].show();
				
				return false
			});
		});
		
		//Deselects files if click on an empty area
		$('upload_area').addEvent('click', function(e){
			//Makes sure clicking on empty area, not on a file
			if( $('upload_area').get('id') == $(e.target).get('id') ) 
				this.deselectFiles();
		}.bind(this));
		
		//Skin the dropdown
		this.skinned_dropdown = new Dropdown($('dd_privacy'), {
			width: 125,
			onSelect: function(value) {
				if( value == "password" )
				{
					$('privacy_password').show();
					$('privacy_password').inject($('edit_area'));
				}
				else
				{
					$('privacy_password').hide();
					$('privacy_password').setProperty('value', '');
				}
				
				this.setPrivacy();
			}.bind(this)
		});
	},
	
	/**
	 * Gets the list of files that were added in previous session
	 */
	getSessionFiles: function()
	{
		var data = '';
		try {
			data = Utility.getLocalStore('vimeo_session_files');
			if (data.substr(0, 1) != '[') {
				data = '[]';
			} 
		} catch (e) {
			data = '[]';
		}
		
		return JSON.decode(data);
	},
	
	getPendingAlbumsSessionFiles: function()
	{
		var data = '';
		try 
		{
			data = Utility.getLocalStore('vimeo_pending_albums');
			if (data.substr(0, 1) != '[' || this.getSessionFiles().length <= 0) 
			{
				data = '[]';
				//If no files in session files, clear out the pending albums
				if( this.getSessionFiles().length <= 0 )
				{
					Utility.setLocalStore('vimeo_pending_albums', JSON.encode('[]'));
				}
			} 
		} 
		catch (e) 
		{
			data = '[]';
		}
		
		return JSON.decode(data);
	},
	
	/**
	 * Updates the session files whenever a change is made
	 */
	updateSessionFiles: function()
	{
		//Update local files info
		var obj = [];
		for (var i = 0; i < this.files.length; i++) {
			var f = this.getFileByOrder(i);
			
			if (!this.hasStatus(i, 'completed') && f.src.exists) {
				obj.push({
					path: f.src.nativePath,
					title: f.title,
					description: f.description,
					tags: f.tags,
					privacy: f.privacy,
					password: f.password,
					addto: f.addto
				});
			}
		}
		
		var pending = []
		//Update pending albums
		$('add_area').getElements('div#addto_Albums .section li').each(function(option, index){
			//Check if it's a pending album
			if( $(option).getElement('input') && $(option).getElement('input').get('id').indexOf('PendingAlbums') >= 0 )
			{
				pending.push({
					id: $(option).getElement('input').get('id').split('PendingAlbums_').join(''),
					title: $(option).getElement('.title').get('text'),
					description: $(option).getElement('.description').get('html')
				});
			}
		});
		
		Utility.setLocalStore('vimeo_session_files', JSON.encode(obj));
		if( pending.length > 0 ) Utility.setLocalStore('vimeo_pending_albums', JSON.encode(pending));
	},
	
	addFile: function(f) 
	{
		adding_files_lightbox.hide();
		
		if (this.is_uploading) {
			error({
				text: 'You must Cancel this upload before you may add more videos to this batch.'				
			});
			return false;
		}
		
		var airfile = false;
		
		var chk = f.toString()
		if (chk.indexOf('File') >= 0) {
			airfile = f;	
		} else {
			airfile = new air.File(f.nativePath);
		}
		
		// File doesn't exist
		if (!airfile.exists) {
			this.updateSessionFiles();
			return false; 
		}

		// Pick apart the file name
		var filename_parts = airfile.name.split('.');
		
		if (filename_parts.length <= 1) 
			return false; // no extension
			
		var ext  = filename_parts.pop().toLowerCase();
		var name = filename_parts.join('.');
		
		// validate video extension
		if (video_extensions.indexOf(ext) == -1) {
			error({
				text: '<div><b>Acceptable video formats</b></div>' + video_extensions.join(', ')
			});
			return false;	
		}
		
		// Check if video is under max allowed upload
		if (vimeo.user_info) {
			if (airfile.size > this.max_file_size) {
				error({
					text: 'Sorry! <b>' + airfile.name + '</b> exceeded the maximum upload size. Please select a video that is under ' + Utility.formatBytes(this.max_file_size) + '.'
				});
				return false;	
			}
		}
		
		if (!this.fileExists(airfile)) {
			var d = new Date();
			
			/**
			 * File properties
			 */
			this.files.push({
				id: hex_md5(d.getTime().toString() + f.nativePath),
				src: airfile,
				uploader: false,
				filename: name,
				ext: ext,
				
				order: this.files.length, // for ordering files
				
				title: name.replace(/\_/g, ' '),
				description: '',
				tags: '',
				privacy: 'anybody',
				password: '',
				addto: [],
				
				video_id: false,
				
				// HTML references
				file_el: false,
				file_index_el: false,
				file_status_el: false,
				file_size_el: false,
				file_link_el: false,
				file_settings_el: false,
				file_links_wrapper_el: false,
				
				// file states
				selected: false,
				progress: 0,
				bytesLoaded: 0,
				bytesTotal: airfile.size,
				
				//time
				start_time: 0,
				end_time: 0,
				elapsed_time: 0,
				avg_rate: 0,
				
				status: 'queued' // waiting, queued, uploading, completed, fail
			});
			
			this.updateSessionFiles();
			
			// Create the file
			var i  = this.files.length - 1;
			
			if (i == 0) 
				$('files').innerHTML = '';
			
			//File Upload Progress Background Element
			this.files[i].file_progress_el = new Element('div', {
				'id': 'file_progressBg_' + this.files[i].id,
				'class': 'progress'
			});
			
			//File Status (Finished, Failed, etc) Element
			this.files[i].file_status_el = new Element('div', {
				'id': 'file_status_' + this.files[i].id,
				'class': 'status',
				'html': 'Waiting',
				'style': 'display:none;'
			});
			
			//Queued Index Number Element
			this.files[i].file_index_el = new Element('div', {
				'id': 'file_index_' + this.files[i].id,
				'class': 'num',
				'html': i + 1
			});
			
			//File Size Element
			this.files[i].file_size_el = new Element('div', {
				'id': 'file_size_' + this.files[i].id,
				'class': 'size',
				'html': Utility.formatBytes(this.files[i].src.size)
			});
			
			//File Name Element
			this.files[i].file_title_el = new Element('div', {
				'id': 'file_title_' + this.files[i].id,
				'class': 'name',
				'html': this.files[i].src.name
			});
			
			//File Link Element
			this.files[i].file_link_el = new Element('a', {
				'id': 'file_link_' + this.files[i].id,
				'class': 'link',
				'html': 'http://vimeo.com'
			});
			
			//File Settings Element
			this.files[i].file_settings_el = new Element('a', {
				'id': 'file_settings_' + this.files[i].id,
				'class': 'link',
				'html': 'settings'
			});
			
			//Links Wrapper Element
			this.files[i].file_links_wrapper_el = new Element(
				'div', {'class': 'links', 'style': 'overflow:hidden; display:none;'}
			).adopt(
				this.files[i].file_link_el
			).adopt(new Element('span', {'html':'&nbsp;('})).adopt(
				this.files[i].file_settings_el
			).adopt(new Element('span', {'html':')'}));
			
			//File Wrapper Element
			this.files[i].file_el = new Element('div', { 
				'id': 'file_el_' + this.files[i].id,
				'class': 'file',
				'events': {
					'click': this.onSelectFile.bind(this)
				}
			}).adopt(
				this.files[i].file_progress_el
			).adopt(
				this.files[i].file_index_el
			).adopt(
				this.files[i].file_title_el
			).adopt(
				this.files[i].file_status_el
			).adopt(
				this.files[i].file_size_el
			).adopt(
				new Element('div', { 'class': 'clear' })
			).adopt(
				this.files[i].file_links_wrapper_el
			);

			
			$('files').adopt(this.files[i].file_el);
			
			// Init the sortable list
			if (!this.sortable_list) {
				this.sortable_list = new Sortables2('files', {
					container: 'files',
					onComplete: this.sortablesOnComplete.bind(this)
				});
			} else { // Add to sortables if already created
				this.sortable_list.addItem(this.files[i].file_el);
			}
			
			this.updateUI();
			
			// Throw a warning when they are going ovelrimit
			if (this.hasStatus(this.files.length - 1, 'overlimit')) {
				error({	errorID: -4	});
			}
			
			this.deselectFiles();
			redrawStatusBar();
			
			return this.files.length - 1;
		}
		
		return false;
	},
	
	sortablesOnComplete: function(ids) {
		for (var i = 0; i < ids.length; i++) {
			var f = this.getFileIndex(ids[i].substring(8));
			this.files[f].order = i;
			this.files[f].file_el.removeEvents('click');
			this.files[f].file_el.addEvent('click', this.onSelectFile.bind(this));
		}
		this.updateUI();
		this.updateSessionFiles();
	},
	
	fileExists: function(f) {
		for (var i = 0; i < this.files.length; i++) {
			if (this.files[i].src.nativePath == f.nativePath) {
				return true;
			}
		}
		
		return false;
	},
	
	/**
	 * Removes any files that are missing after adding to duplo
	 */
	removeMissingFiles: function() {
		for (var i = 0; i < this.files.length; i++) {
			if (!this.files[i].src.exists) {
				this.removeFile(i);
				error({ errorID: -5, text: 'File is missing and was removed' })
			}
		}
	},
	
	createAddToArea: function()
	{
		// debug('createAddToArea');
		
		if(!this.add_area)
		{
			air.NativeApplication.nativeApplication.removeEventListener(air.Event.ACTIVATE, this.refreshAddToSection);
			air.NativeApplication.nativeApplication.addEventListener(air.Event.ACTIVATE, this.refreshAddToSection.bind(this));
		}
		this.drawAddTo();
	},
	
	drawAddTo: function()
	{
		// debug('drawAddTo');
		
		if(!this.add_area) this.add_area = $('add_area');
		this.add_area.empty();
		
		//Draw Groups
		this.add_area.grab( this.drawAddToSection('Groups', vimeo.user_groups) );
		
		//Draw Channels
		this.add_area.grab( this.drawAddToSection('Channels', vimeo.user_channels) );
		
		//Draw Albums
		this.add_area.grab( this.drawAddToSection('Albums', vimeo.user_albums) );
		
		//Draw Pending Albums
		var pending = this.getPendingAlbumsSessionFiles();
		pending.each(function(obj, index){
			this.createPendingAlbum(obj.id, obj.title, obj.description);
		}.bind(this));
		
		//Create new album button
		this.add_area.getElement('div#addto_Albums').grab((new Element('a', {
			'id': 'add_album_link',
			'class': 'faux_link', 
			'html': 'New album',
			'href': '#add_album_form'
		})));
		$('add_album_link').addEvent('click', function(e){
				$('add_album_form').show();
				$(e.target).hide();
				$('add_album_title').focus();
		}.bind(this));
		
		//Create new album form
		this.add_area.getElement('div#addto_Albums').grab(new Element('form', {
			'id'	: 'add_album_form',
			'style'	: 'display: none;',
			'html'	: '<fieldset>\
							<label for="add_album_title">New Album Name</label>\
							<input id="add_album_title" class="field" />\
						</fieldset>\
						<fieldset>\
							<label for="add_album_description">Album Description</label>\
							<textarea id="add_album_description" rows="4"></textarea>\
						</fieldset>\
						<fieldset>\
							<input id="add_album_submit" type="submit" value="Add" />\
							&nbsp;or&nbsp;\
							<a class="faux_link" href="#add_album_link">Cancel</a>\
						</fieldset>'
		}));
		//Submit button
		$('add_album_form').addEvent('submit', function(e){
			e.preventDefault();
			
			var uid = 'xxx'+this.getPendingAlbumsSessionFiles().length;
			this.createPendingAlbum(uid, $('add_album_title').get('value'), $('add_album_description').get('html'));
			
			//Clear the fields
			$('add_album_title').set('value', '');
			$('add_album_description').set('value', '');
			
			$('add_album_form').hide();
			$('add_album_link').show();
		}.bind(this));
		//Cancel button
		$('add_album_form').getElement('a.faux_link').addEvent('click', function(e){
			$('add_album_form').hide();
			$('add_album_link').show();
		});
	},
	
	drawAddToSection: function(name, data)
	{
		var wrapper = new Element('div', {'id': 'addto_'+name});
		
		var header = new Element('h2', {
			'html': 'Add to ' + name
		});
		
		var section = new Element('ul', {
			'class': 'section'
		});
		
		if( data[0].id )
		{
			var option;
			data.each(function(item, index){
				section.grab(this.createAddToOption( name, item.id, (item.name || item.title) ));
			}.bind(this));
		}
		else
		{
			var emptyState = new Element('li',{
				'html': '<div class="bubble">'+ data[0]+'</div>'
			});
			if( emptyState.getElement('a') )
			{
				emptyState.getElement('a').addEvent('click', function(e){
					Utility.goToURL( $(e.target).get('href'));
					e.preventDefault();
				}.bind(this));
			}
			section.grab(emptyState)
		}
		
		
		wrapper.grab(header);
		wrapper.grab(section);
		
		return wrapper;
	},
	
	createAddToOption: function(sectionname, id, title)
	{
		var option = new Element('li', {
			'html': '<input type="checkbox" id="'+sectionname+'_'+id+'" /> <label for="'+sectionname+'_'+id+'">' + title + '</label>'
		});

		option.addEvent('change', function(e){
			this.setAddTo( $(e.target).get('id'), $(e.target).get('checked') );
		}.bind(this));
		
		return option;
	},
	
	//Creates an album on the front end, but it hasn't been created on the back end yet
	createPendingAlbum: function(id, title, description)
	{
		var option = this.createAddToOption('PendingAlbums', id, title);
		option.grab(new Element('span', {
			'class': 'title',
			'style': 'display: none;',
			'html': title
		}));
		option.grab(new Element('span', {
			'class': 'description',
			'style': 'display: none;',
			'html': description
		}));
		
		//Add to the DOM
		$('add_area').getElement('div#addto_Albums .section').grab( option );
		
		option.getElement('input[type=checkbox]').set('checked', 'true');
		option.fireEvent('change', {target:option.getElement('input[type=checkbox]')});
	},
	
	refreshAddToSection: function()
	{
		// debug('refreshAddToSection');
		
		var batch_ajax = new BatchAJAX(this.drawAddTo.bind(this));
		batch_ajax.addAJAX(vimeo.getGroups());
		batch_ajax.addAJAX(vimeo.getChannels());
		batch_ajax.addAJAX(vimeo.getAlbums());
	},
	
	updateUI: function()
	{
		this.removeMissingFiles();
		
		if (this.is_uploading) { 
			$('add_files').hide();
			$('remove_videos').hide();
			$('upload_status_area').hide();
			$('progress_area').show();
			
			$('clear_finished_btn').show();
			$('clear_finished_btn').disable();
		}
		else {
			$('add_files').show();
			$('remove_videos').show();
			$('upload_status_area').show();
			$('progress_area').hide();
			
			$('clear_finished_btn').hide();
		}
		
		var are_files_selected = false;
		var total_selected_files = 0;
		var visual_i = 0;
		var cur_queued_space = 0;
		
		this.total_allowable_uploads = 0;
		this.total_selectable_files = 0;
		
		// Using the actual order in HTML to figure out the sorting
		var ordered_files = $('files').getChildren();
		var retry_files = 0;	//all the files should be retried, it means all of the files have failed or been canceled
		
		for (var i = 0; i < ordered_files.length; i++) {
			var cur_file = this.getFileById(ordered_files[i].id.substring(8));
			if (!cur_file) continue;
			
			cur_file.order = i; // just set this again
			var index = this.getFileIndex(cur_file.id);
			
			if (cur_file.selected) {
				are_files_selected = true;
				total_selected_files++;
			}
			
			if (cur_file.file_el) {
				// Calculate queue size
				if( !this.hasStatus(index, 'completed') ) cur_queued_space += cur_file.src.size;
				// debug(Utility.formatBytes(cur_queued_space) + " > " + Utility.formatBytes(this.disk_usage.max - this.disk_usage.used) + " (" + Utility.formatBytes(this.disk_usage.max) + " - " + Utility.formatBytes(this.disk_usage.used) + ")" );
				
				if (cur_queued_space > this.disk_usage.max - this.disk_usage.used) {
					// debug('OVERLIMIT!!!!!!!!!!!!!!!');
					this.upload_status = 'overlimit';
					this.updateFileStatus(index, this.upload_status);
				}
				else {
					//If it's none of these statuses, set to queued by default
					if (!this.is_uploading && !this.hasStatus(index, 'completed') && 
						!this.hasStatus(index, 'failed') && 
						!this.hasStatus(index, 'canceled') &&
						!this.hasStatus(index, 'skipped'))
					{
						this.upload_status = 'queued';
						this.updateFileStatus(index, this.upload_status);
					}
				}
				
				cur_file.file_index_el.innerHTML = visual_i + 1;
				
				visual_i++;
			}
			
			switch (cur_file.status) {
				case 'completed':
					$('clear_finished_btn').show();
					$('clear_finished_btn').enable();
					
					if (this.is_uploading) this.total_allowable_uploads++;
					break;
					
				case 'overlimit': // When uploading, hide overlimit files
					this.total_selectable_files++;
					
					if (this.is_uploading )
						cur_file.file_el.hide();
					else
						cur_file.file_el.show();
					break;
				
				//If all of the files are any one of these statuses, set the upload text to "retry"
				case 'failed':
				case 'canceled':
				case 'skipped':
					retry_files++;
				default: 
					this.total_selectable_files++;
					this.total_allowable_uploads++;
					break;
			}
		}
		
		var str = this.total_allowable_uploads > 1 ? 'all ' : '';
		if( retry_files == ordered_files.length )
			$('upload_btn').value = 'Retry ' + str + this.total_allowable_uploads + ' video' + (this.total_allowable_uploads > 1 ? 's' : '');
		else
			$('upload_btn').value = 'Upload ' + str + this.total_allowable_uploads + ' video' + (this.total_allowable_uploads > 1 ? 's' : '');
			
		if( upload_schedule && upload_schedule.getStartTime() ) $('upload_btn').value += ' now';
		
		if (this.total_selectable_files > 0) {
			$('select_all_videos').enable();
			$('remove_videos').enable();
		} else {
			$('select_all_videos').disable();
			$('remove_videos').disable();
		}
		
		if (this.total_allowable_uploads > 0) {
			$('upload_btn').show();
			if(upload_schedule) upload_schedule.show();
		}
		else {
			$('edit_video_info_title').innerHTML = 'Edit video info';
			$('upload_btn').hide();
			if(upload_schedule) upload_schedule.hide();
		}
		
		if (are_files_selected) {
			$('edit_video_info_title').removeClass('disabled');
			$('edit_video_info_title').innerHTML = total_selected_files == 1 ? 'Editing video' : 'Editing ' + total_selected_files + ' video' + (total_selected_files > 1 ? 's' : '')
			$('edit_area').enable();
			
			$('add_to_info_title').innerHTML = total_selected_files == 1 ? 'Add to' : 'Add ' + total_selected_files + ' videos to*';
			$('add_to_info_title').removeClass('disabled');
			$('add_area').enable();
		} 
		else {
			$('edit_video_info_title').fireEvent('click');
			
			$('edit_video_info_title').addClass('disabled');
			$('edit_video_info_title').innerHTML = 'Edit video info';
			$('edit_area').disable('e7e7de');
			
			$('add_to_info_title').innerHTML = 'Add to';
			$('add_to_info_title').addClass('disabled');
			$('add_area').disable('e7e7de');
		}
		
		if (this.files.length > 0) {
			$('files_empty_state').hide();
		}
		else {
			var empty_state = $('files_empty_state');
			empty_state.show();
			empty_state.setStyle('margin-top', ($('upload_area').getHeight() / 2 - empty_state.getHeight() / 2) + 'px');
		}
		
		$('promo_10x_upload_space').hide();
		if (this.files.length > 0 && vimeo.user_info && vimeo.user_info.is_plus != 1) {
			$('promo_10x_upload_space').show();
		}
		
		//Update Add To Area
		//check privacy
		var isPrivate = false;
		for (var i = 0; i < this.files.length; i++)
		{
			if (this.files[i].selected) 
			{
				if(!isPrivate) isPrivate = this.files[i].privacy != 'anybody' && this.files[i].privacy != '';
			}
		}
		
		this.updateInfo();
		this.drawProgress();
		this.redraw();
		redrawStatusBar();
	},
	
	updateInfo: function()
	{
		var unedited_str = '[[UNEDITED]]';
		
		var title_str 	 = unedited_str;
		var desc_str  	 = unedited_str;
		var tag_str  	 = unedited_str;
		var privacy_str  = unedited_str;
		var password_str = unedited_str;
		var addto_str	 = unedited_str;
		
		var selected_num = 0;
		
		for (var i = 0; i < this.files.length; i++) {
			if (this.files[i].selected) 
			{
				selected_num++;
				
				if (title_str == unedited_str) {
					title_str = this.files[i].title;
				} 
				else if (this.files[i].title != title_str) {
					title_str = '';
				}
				
				if (desc_str == unedited_str) {
					desc_str = this.files[i].description;
				} 
				else if (this.files[i].description != desc_str) {
					desc_str = '';
				}		
				
				if (tag_str == unedited_str) {
					tag_str = this.files[i].tags;
				} 
				else if (this.files[i].tags != tag_str) {
					tag_str = '';
				}
				
				if (privacy_str == unedited_str) {
					privacy_str = this.files[i].privacy;
				}
				else if (this.files[i].privacy != privacy_str) {
					privacy_str = 'anybody';
				}
				
				if (password_str == unedited_str) {
					password_str = this.files[i].password;
				}
				else if (this.files[i].password != password_str) {
					password_str = '';
				}
				
				//add to hasn't been edited
				if (addto_str == unedited_str) {
					addto_str = this.files[i].addto ? this.files[i].addto.toString() : '';
				}
				//multiple files selected, clear out the selection
				else if (this.files[i].addto != addto_str) {
					addto_str = '';
				}
			}
		}
		
		$('txt_title').value 		= title_str == unedited_str ? '' : title_str;
		$('txt_description').value  = desc_str  == unedited_str ? '' : desc_str;
		$('txt_tags').value  		= tag_str   == unedited_str ? '' : tag_str;
		
		$('dd_privacy').value 		= privacy_str == unedited_str ? 'anybody' : privacy_str;
		//Update the skinned dropdown
		if( this.skinned_dropdown ) this.skinned_dropdown.updateSkin();
		
		$('privacy_password').value	= password_str == unedited_str ? '' : password_str;
		
		$('add_area').getElements('.section li input[type=checkbox]').each(function(item){
			//Current checkbox's id is in addto str
			if( addto_str.indexOf(item.get('id')) > -1 ) {
				$(item).set('checked', true);
			}
			//Id is not in addto str
			else {
				$(item).set('checked', false);
				if( selected_num > 1 ) this.setAddTo($(item).get('id'), false);
			}
		}.bind(this));
		
		// this.updateSessionFiles();
	},
	
	setTitle: function()
	{
		for (var i = 0; i < this.files.length; i++) {
			if (this.files[i].selected) {
				this.files[i].title = $('txt_title').value;
			}
		}
		this.updateSessionFiles();
	},
	
	setDescription: function()
	{
		for (var i = 0; i < this.files.length; i++) {
			if (this.files[i].selected) {
				this.files[i].description = $('txt_description').value;
			}
		}
		this.updateSessionFiles();
	},
	
	setTags: function()
	{
		for (var i = 0; i < this.files.length; i++) {
			if (this.files[i].selected) {
				this.files[i].tags = $('txt_tags').value;
			}
		}
		this.updateSessionFiles();
	},
	
	setPrivacy: function()
	{
		var isPrivate = false;
		var privacy_msg = $('privacy_msg');
		for (var i = 0; i < this.files.length; i++)
		{
			if (this.files[i].selected) 
			{
				this.files[i].privacy = $('dd_privacy').value;
				
				//Update Add To Area
				//check privacy
				if(!isPrivate) isPrivate = this.files[i].privacy != 'anybody' && this.files[i].privacy != '';

				//Add private video message
				if(isPrivate)
				{
					if( !privacy_msg )
					{
						privacy_msg = new Element('div',{
							'id': 'privacy_msg',
							'class': 'bubble',
							'html': '<strong>This video is private.</strong> When you add this private video to a Channel or Group that is public, this video will become visible in that Channel or Group.'
						});
						privacy_msg.inject($('add_area'), 'top');
					}
					privacy_msg.show();
				}
				else
				{
					if( privacy_msg ) privacy_msg.hide();
				}
			}
			
			//Check privacy
			if( this.files[i].privacy != 'anybody' && this.files[i].privacy != '' )
			{
				//File is private
				if( !$(this.files[i].file_el).hasClass('private') )
				{
					$(this.files[i].file_el).addClass('private');
				}
			}
			else
			{
				//Not private
				$(this.files[i].file_el).removeClass('private');
			}
		}
		this.updateSessionFiles();
	},
	
	setPassword: function()
	{
		for (var i = 0; i < this.files.length; i++) {
			if (this.files[i].selected) {
				this.files[i].password = $('privacy_password').value;
			}
		}
		this.updateSessionFiles();
	},
	
	setAddTo: function(id, include)
	{
		for (var i = 0; i < this.files.length; i++) {
			if (this.files[i].selected) {
				if(include) this.files[i].addto.include(id);
				else this.files[i].addto.erase(id);
			}
		}
		
		this.updateSessionFiles();
	},
	
	// This is called whenever the window is resized
	redraw: function()
	{
		for (var i = 0; i < this.files.length; i++) {
			// Set width of the file title when resizing
			var w = this.files[i].file_el.getWidth();
			var sw = this.files[i].file_status_el.getWidth();
			this.files[i].file_title_el.setWidth(w - sw - 117, true);
		}
	},
	
	resetDiskSpace: function()
	{
		this.disk_usage = false;
	},
	
	getTotalQueuedFileSize: function()
	{
		var size = 0;
		for (var i = 0; i < this.files.length; i++) {
			if (!this.hasStatus(i,'completed') )
				size += this.files[i].src.size.toInt();
		}
		return size;
	},
	
	getOverlimitFileSize: function()
	{
		var size = 0;
		for (var i = 0; i < this.files.length; i++) {
			if (this.hasStatus(i,'overlimit') )
				size += this.files[i].src.size.toInt();
		}
		return size;
	},
	
	getQueuedFileSize: function()
	{
		var size = 0;
		for (var i = 0; i < this.files.length; i++) {
			if (!this.hasStatus(i,'completed') && !this.hasStatus(i,'overlimit') )
				size += this.files[i].src.size.toInt();
		}
		return size;
	},
	
	getFileByOrder: function(order) {
		for (var i = 0; i < this.files.length; i++) {
			if (order == this.files[i].order)
				return this.files[i];
		}
	},
	
	getFileById: function(id) {
		for (var i = 0; i < this.files.length; i++) {
			if (id == this.files[i].id)
				return this.files[i];
		}
	},
	
	getFileIndex: function(id) {
		for (var i = 0; i < this.files.length; i++) {
			if (id == this.files[i].id)
				return i;
		}
	},
	
	lockUploadControls: function()
	{
		$('resume_btn').disable();
		$('pause_btn').disable();
		$('cancel_btn').disable();
		
		//Controls unresponsive if haven't been unlocked in 1 minute
		this.lockControlsTimer = this.onControlsUnresponsive.delay(this.controlsUnresponsiveDelay);
	},
	
	unlockUploadControls: function()
	{
		$('resume_btn').enable();
		$('pause_btn').enable();
		$('cancel_btn').enable();
		
		//Cancel unresponsive timer
		this.lockControlsTimer = $clear(this.lockControlsTimer);
	},
	
	// Fires when the controls have been locked for too long
	onControlsUnresponsive: function()
	{
		error({ errorID: -11, text: 'Controls unresponsive.' });
	},
	
	deselectFile: function(i) {
		this.files[i].selected = false;
		this.files[i].file_el.removeClass('file_selected');
	},
	
	deselectFiles: function()
	{
		for (var i = 0; i < this.files.length; i++) {
			this.deselectFile(i);
		}
		this.updateUI();
	},
	
	selectFile: function(i) {
		if (!this.hasStatus(i, 'completed') && (!this.hasStatus(i, 'overlimit') || !this.is_uploading)) {
			this.files[i].selected = true;
			this.files[i].file_el.addClass('file_selected');
		}
		
		this.updateUI();
	},
	
	onSelectFile: function(e) {
		var name = '';
		var id 	 = '';
		var t    = e.target;
		
		// sometimes click event calls children and not parent. traverse upwards
		while( name != 'file_el_') {
			name = t.id.substring(0, 8);
			id 	 = t.id.substring(8);
			t    = t.getParent();
		}
		
		var f 		= this.getFileById(id);
		var index 	= this.getFileIndex(f.id);

		// check if file was moved
		if (!f.src.exists) {
			this.removeFile(index);
			return false;
		}
		
		if (f.status == 'completed') return false; // currenty uploading this file
		
		// Holding CONTROL/COMMAND
		if (e.control) {
			if (f.selected) {
				this.deselectFile(index);
			} else {
				this.selectFile(index);
			}
		}
		// Holding SHIFT
		else if (e.shift && this.last_selected_file >= 0 && this.last_selected_file != index) {
			var start = f.order < this.files[this.last_selected_file].order ? f.order : this.files[this.last_selected_file].order;
			var end   = f.order > this.files[this.last_selected_file].order ? f.order : this.files[this.last_selected_file].order;
			
			for (var i = 0; i < this.files.length; i++) {
				if (this.files[i].order >= start && this.files[i].order <= end ) 
					this.selectFile(i);	
				else
					this.deselectFile(i);	
			}
		} // Normal click
		else {
			this.deselectFiles();
			this.selectFile(index);
			
			this.last_selected_file = index;
		}
		
		this.updateUI();
	},
	
	selectAllFiles: function () {
		for (var i = 0; i < this.files.length; i++) {
			this.selectFile(i);
		}
	},
	
	removeFile: function(i) {
		// cancel any uploading
		if (this.files[i].uploader) 
			this.files[i].uploader.cancel();
		
		// remove from sortables
		this.sortable_list.removeItem(this.files[i].file_el);
		
		// destroy the file element
		this.files[i].file_el.destroy(); 
		this.files[i].src = false;
		
		var cur_file = false;
		if (this.is_uploading) 
			cur_file = this.getFileByOrder(this.cur_file_pos);
		
		// remove from array
		this.files.erase( this.files[i] );
		
		this.updateUI(); // update ordering
		
		this.updateSessionFiles(); // update session
		
		// Set the new position of the current file
		if (this.is_uploading) 
			this.cur_file_pos = cur_file.order;
	},
	
	removeCompletedFiles: function()
	{
		var finished = true;
		while(1) {
			finished = true;
			 
			for (var i = 0; i < this.files.length; i++) {
				if (this.hasStatus(i,'completed')) {
					this.removeFile(i);
					finished = false;
					break;
				}
			}
			if (finished ) break;
		}
		
		this.updateUI();
	},
	
	removeSelectedFiles: function()
	{
		var finished = true;
		while (1) {
			finished = true;
			 
			for (var i = 0; i < this.files.length; i++) {
				if (this.files[i].selected) {
					this.removeFile(i);
					finished = false;
					break;
				}
			}
			if (finished ) break;
		}
		
		this.updateUI();
		redrawStatusBar();
	},
	
	isSelected: function()
	{
		for (var i = 0; i < this.files.length; i++) {
			if (this.files[i].selected ) return true;
		}
		return false;
	},
	
	resumeUpload: function()
	{
		this.is_paused = false;
		
		$('pause_btn').show();
		$('resume_btn').hide();
		
		var cur_file = this.getFileByOrder(this.cur_file_pos);
		var i 	 	 = this.getFileIndex(cur_file.id);
		
		this.files[i].uploader.resume();
		this.drawProgress();
		
		VimeoLogger.log('upload_resume');
	},
	
	pauseUpload: function()
	{
		if (this.is_paused === false) {
			this.is_paused = true;
			this.upload_status = 'paused';
			
			$('pause_btn').hide();
			$('resume_btn').show();
			
			var cur_file = this.getFileByOrder(this.cur_file_pos);
			var i 	 	 = this.getFileIndex(cur_file.id);
			
			this.updateFileStatus(i, this.upload_status);
			
			this.files[i].uploader.pause();
			this.drawProgress();
			
			VimeoLogger.log('upload_paused');
		}
	},
	
	cancelUpload: function()
	{
		this.upload_status = 'idle';
		
		if (!this.is_uploading) // already canceled
			return false;
			
		if (this.sortable_list) 
			this.sortable_list.unlock();
		
		this.upload_rate = 0;
		this.upload_last_bytes_loaded = 0;
		this.total_videos_completed = 0;
		
		for (var i = 0; i < this.files.length; i++) {
			this.cancelSingleUpload(i);
		}
		
		this.is_uploading = false;
		this.is_paused    = false;
		redrawWindow();
		
		if(upload_schedule) upload_schedule.enable();
		
		VimeoLogger.log('upload_canceled', { 
			finished: this.total_videos_completed,
			total: this.total_allowable_uploads
		});
	},
	
	cancelSingleUpload: function( i )
	{
		// check if file disappeared
		if (this.files[i] && !this.files[i].src.exists) {
			this.removeFile(i);
			break;
		}
		
		if (this.files[i].uploader) {
			this.files[i].uploader.cancel();
			// this.files[i].uploader.pause();
		}
		
		//If it wasn't any of these statuses, then it was just straight up canceled
		if (!this.hasStatus(i,'completed') && 
			!this.hasStatus(i, 'overlimit') && 
			!this.hasStatus(i,'skipped')) 
		{
			if( this.files[i].bytesLoaded > 0 ) this.upload_status = 'canceled';
			else this.upload_status = 'queued';
			
			this.files[i].bytesLoaded = 0;
			this.updateFileStatus(i, this.upload_status);
		}
		
		if (this.hasStatus(i, 'completed')) 
			this.total_videos_completed++;
	},
	
	/**
	 * Restarts the upload process.
	 * @param current [Boolean] - If true, only restarts the current upload, not all the uploads.
	 **/
	restartUpload: function(current)
	{
		if( current && this.cur_file_pos )
		{
			this.cancelSingleUpload(this.cur_file_pos);
			this.updateUI();
			this.uploadCurVideo();
		}
		else
		{
			this.cancelUpload();
			this.startUpload();
		}
	},
	
	startUpload: function()
	{
		if( upload_schedule && upload_schedule.getStartTime() )
		{
			var scheduled_time_el = has_schedule_lightbox.content.getElement('.scheduled_time');
			scheduled_time_el.empty();
			scheduled_time_el.set('html', upload_schedule.getTimeRemaining());
			has_schedule_lightbox.show();
			return false;
		}
		
		// Updating here to check if any files went missing before we upload
		this.updateUI();
		
		has_schedule_lightbox.close();	//just in case it didn't close
		
		if (this.files.length == 0) {
			return false;
		}
		
		this.removeCompletedFiles();
		this.sortable_list.lock();
		this.cur_file_pos = 0;
		
		showProgressBar();
		
		this.prev_rate_data = Utility.getLocalStore('vimeo_rate_data') != '' ? JSON.decode(Utility.getLocalStore('vimeo_rate_data')) : null;
		this.drawProgress();
		this.is_uploading = true;
		this.is_paused	  = false;
		
		// Disable the file list and show the statuses
		for (var i = 0; i < this.files.length; i++) {
			if (!this.hasStatus(i, 'completed')) {
				this.upload_status = 'waiting';
				this.updateFileStatus(i, this.upload_status);
			}
		}
		
		this.deselectFiles();
		this.updateUI();

		this.lockUploadControls();
		
		this.uploadCurVideo.delay(500, this);
		
		if(upload_schedule) upload_schedule.disable();
		
		VimeoLogger.log('upload_start', {
			total: this.total_allowable_uploads
		});
	},
	
	uploadCurVideo: function()
	{
		this.sortable_list.lock();
		
		$('pause_btn').show();
		$('resume_btn').hide();
		$('upload_btn').hide();

		var cur_file = this.getFileByOrder(this.cur_file_pos);
		var index 	 = this.getFileIndex(cur_file.id);

		// File is too big. skip it!
		if (cur_file.status == 'overlimit') {
			this.onUploadComplete();
			return false;
		}
		
		this.upload_status = 'starting';
		this.updateFileStatus(index, this.upload_status);
		this.updateUI();
		
		var cur_file = this.getFileByOrder(this.cur_file_pos);
		if (!cur_file.uploader) 
		{
			cur_file.uploader = new Uploader(cur_file.src, {
				uid: cur_file.id,
				onProgress: 	this.onProgress.bind(this),
				onComplete: 	this.onUploadComplete.bind(this),
				onNearComplete: this.onNearComplete.bind(this),
				onLock: 		this.onLock.bind(this),
				onUnlock: 		this.onUnlock.bind(this),
				onError: 		this.onError.bind(this)
			});
		}
		
		cur_file.start_time = (new Date()).getTime();
		cur_file.uploader.upload();
	},
	
	hasStatus: function(i, type) {
		return this.files[i].status == type;
	},
	
	updateFileStatus: function(i, type, arg) {
		this.files[i].status = type;
		this.files[i].file_el.className = 'file' + (this.files[i].selected ? ' file_selected' : '');
		this.files[i].file_el.addClass('file_' + type);
		this.files[i].file_status_el.show();
		this.files[i].file_progress_el.show();
		
		switch(type) {
			case 'uploading':
				this.files[i].file_status_el.innerHTML = (arg < 0 ? 0 : Math.floor(arg)) + '%';
				this.files[i].file_progress_el.setWidth( (arg/100) * this.files[i].file_el.getWidth() );
				this.unlockUploadControls();
				break;
				
			case 'completed':
				this.files[i].file_el.addClass('file_disabled');
				this.files[i].file_status_el.innerHTML = 'Done!';
				this.files[i].file_title_el.innerHTML = this.files[i].title == '' ? 'Untitled' : this.files[i].title;
				this.files[i].file_progress_el.hide();
				
				//Show links
				this.files[i].file_links_wrapper_el.show();
				
				//Set link
				this.files[i].file_link_el.set('html', 'http://' + VIMEO_URL + '/' + this.files[i].video_id);
				this.files[i].file_link_el.addEvent('click', function() {
					Utility.goToURL('http://' + VIMEO_URL + '/' + this.video_id);
					VimeoLogger.log('video_link_click', {
						video_id: this.video_id
					});
				}.bind(this.files[i]));
				
				//Set settings
				this.files[i].file_settings_el.addEvent('click', function() {
					Utility.goToURL('http://' + VIMEO_URL + '/' + this.video_id + '/settings');
					VimeoLogger.log('video_settings_click', {
						video_id: this.video_id
					});
				}.bind(this.files[i]));
				
				this.deselectFile(i);
				this.updateUI();
				break;
				
			case 'starting':
				this.files[i].file_status_el.innerHTML = 'Starting';
				this.files[i].file_progress_el.setWidth(0);
				break;
				
			case 'waiting':
				this.files[i].file_status_el.innerHTML = 'Waiting';
				this.files[i].file_status_el.show();
				this.files[i].file_progress_el.setWidth(0);
				break;
				
			case 'queued':
				this.files[i].file_status_el.hide();
				this.files[i].file_status_el.innerHTML = '';
				this.files[i].file_progress_el.setWidth(0);
				this.files[i].file_progress_el.hide();
				this.files[i].file_progress_el.setWidth(0);
				break;
				
			case 'finishing':
				this.files[i].file_status_el.innerHTML = 'Finishing';
				this.files[i].file_status_el.show();
				break;
			
			case 'saving':
				this.files[i].file_status_el.innerHTML = 'Saving';
				this.files[i].file_status_el.show();
				break;
			
			case 'adding':
				this.files[i].file_status_el.innerHTML = 'Adding';
				this.files[i].file_status_el.show();
				break;
			
			case 'paused':
				this.files[i].file_status_el.innerHTML = 'Paused';
				break;
				
			case 'failed':
				this.files[i].file_status_el.innerHTML = 'Failed';
				this.files[i].file_progress_el.hide();
				this.files[i].file_progress_el.setWidth(0);
				break;
				
			case 'overlimit':
				this.files[i].file_status_el.innerHTML = 'Too big';
				this.files[i].file_progress_el.hide();
				this.files[i].file_progress_el.setWidth(0);
				break;
				
			case 'canceled':
				this.files[i].file_status_el.innerHTML = 'Canceled';
				this.files[i].file_progress_el.hide();
				this.files[i].file_progress_el.setWidth(0);
			break;
			
			case 'skipped':
				this.files[i].file_status_el.innerHTML = 'Skipped';
				this.files[i].file_status_el.show();
				this.files[i].file_progress_el.hide();
				this.files[i].file_progress_el.setWidth(0);
			break;
		}
		
		//Check privacy
		if( this.files[i].privacy != 'anybody' && this.files[i].privacy != '' )
		{
			//File is private
			if( !$(this.files[i].file_el).hasClass('private') )
			{
				$(this.files[i].file_el).addClass('private');
			}
		}
		else
		{
			//Not private
			$(this.files[i].file_el).removeClass('private');
		}
	},
	
	onUploadComplete: function(uid, json) {
		debug('onUploadComplete', uid, json, '=================');
		var f = this.getFileById(uid);
		var i = this.getFileIndex(uid);
		
		// Don't do anything to files that are overlimit. Just skip
		if (f.status != 'overlimit') {
			if (!Utility.verifyJSONResponse(json)) {
				this.upload_status = 'failed';
				this.updateFileStatus(i, this.upload_status);
			}
			else {
				f.video_id = json.ticket.video_id;
				this.saveVideo.delay(1500, this, [i, f]);
			}
		}
		
		//Calculate average rate of the file upload
		f.elapsed_time = f.end_time - f.start_time;
		f.avg_rate = f.bytesTotal / (f.elapsed_time/1000);
		
		Utility.setLocalStore('vimeo_rate_data', JSON.encode({start_time: f.start_time, end_time: f.end_time, elapsed_time: f.elapsed_time, avg_rate: f.avg_rate}));
		
		// Update used storage
		this.disk_usage.used = this.disk_usage.used.toInt() + f.src.size.toInt();
		
		// this.onSaveComplete(f);
	},
	
	/**
	 * Save the video info
	 */
	saveVideo: function(i, f) {
		debug('saveVideo()', i, f, '=================');
		
		this.upload_status = 'saving';
		this.drawProgress();
		this.updateFileStatus(i, this.upload_status);
		
		var batch_ajax = new BatchAJAX( this.onSaveComplete.pass([i, f], this) );
		batch_ajax.addAJAX( vimeo.setTitle(this.files[i].video_id, this.files[i].title, false) );
		batch_ajax.addAJAX( vimeo.setDescription(this.files[i].video_id, this.files[i].description) );
		batch_ajax.addAJAX( vimeo.setTags(this.files[i].video_id, this.files[i].tags) );
		batch_ajax.addAJAX( vimeo.setPrivacy(this.files[i].video_id, this.files[i].privacy, null, this.files[i].password) );
	},
	
	onSaveComplete: function(i, f)
	{
		debug('onSaveComplete', i, f, '=================');
		
		this.upload_status = 'adding';
		this.drawProgress();
		this.updateFileStatus(i, this.upload_status);
		
		if( this.files[i].addto && this.files[i].addto != '' )
			vimeo.setAddTo( this.files[i].video_id, this.files[i].addto, this.onAddingComplete.pass([i, f], this), this.onAddingFailed.pass([i, f], this) );
		else
			this.onAddingComplete(i, f);
	},
	
	onAddingComplete: function(i, f)
	{
		debug('onAddingComplete', i, f, '=================');
		
		this.upload_status = 'completed';
		this.updateFileStatus(i, this.upload_status);
		this.updateSessionFiles();
		
		this.unlockUploadControls();
		
		this.sortable_list.removeItem(f.file_el);
		
		this.uploadNextFile();
	},
	
	onAddingFailed: function(i, f)
	{
		this.onAddingComplete(i, f);
		// debug('onAddingFailed', i, f, '=================');
		// 
		// metadata_failed_lightbox.show();
		// 
		// $(metadata_failed_lightbox.content).getElement('.blue').addEvent('click', function(e){
		// 	Utility.goToURL('http://' + VIMEO_URL + '/' + this.files[i].video_id + '/settings');
		// }.bind(this));
		// 
		// metadata_failed_lightbox.addEvent('windowClose', function(){
		// 	this.onAddingComplete(i, f);
		// }.bind(this));
	},
	
	skipFile: function(  )
	{
		this.files[this.cur_file_pos].bytesLoaded = 0;
		this.files[this.cur_file_pos].progress = 0;
		
		var cur_file = this.getFileByOrder(this.cur_file_pos);
		cur_file.uploader.cancel();
		
		this.upload_status = 'skipped';
		this.updateFileStatus(this.cur_file_pos, this.upload_status);
		
		this.uploadNextFile()
	},
	
	uploadNextFile: function(  )
	{
		debug('uploadNextFile');
		// Start next upload
		if (this.cur_file_pos + 1 < this.total_allowable_uploads) 
		{
			this.cur_file_pos++;
			this.uploadCurVideo();
		} // Done uploading
		else 
		{
			// log how many vidz were uploaded (this may be slightly inaccurate since people can Clear Finished)
			VimeoLogger.log('batch_finished', { 
				num_videos: this.total_allowable_uploads
			});
			
			this.sortable_list.unlock(); // allow the files to be sorted again
			this.is_uploading = false;
			this.is_paused    = false;
			this.updateUI();
			redrawWindow();
			
			if(upload_schedule) upload_schedule.enable();
		}
	},
	
	/**
	 * Tells us when a video finished uploading and is going through verifying process (lock controls)
	 */
	onNearComplete: function(uid) { // needs renamed
		this.upload_status = 'finishing';
		
		this.lockUploadControls();
		
		var curIndex = this.getFileIndex(uid);
		this.updateFileStatus(curIndex, this.upload_status);
		this.drawProgress(curIndex);
	},
	
	onError: function(uid, err) {
		this.upload_status = 'error';
		/*onError(err);
		
		if (err.errorID == 1) { // file was moved, remove it
			//this.cancelUpload();
			this.removeFile(this.getFileIndex(uid));
			this.uploadCurVideo();
		}*/
	},
	
	/**
	 * Whenever the uploader has to lock the system it will call this first
	 */
	onLock: function()
	{
		this.system_locked = true;
	},
	
	/**
	 * System unlocked
	 */
	onUnlock: function()
	{
		this.system_locked = false;
	},
	
	onProgress: function(uid, e) {
		this.upload_status = 'uploading';
		
		var i = this.getFileIndex(uid);
		
		if (!this.hasStatus(i, 'completed')) { 
			var f = this.getFileById(uid);
			
			f.bytesLoaded = e.bytesLoaded < 0 ? 0 : e.bytesLoaded;
			
			var p = e.bytesLoaded / e.bytesTotal;
			this.updateFileStatus(i, this.upload_status, p * 100);
					
			this.drawProgress();
			
			//If finished, store the end time
			if( p == 1 ) f.end_time = (new Date()).getTime();
		}
	},
	
	// upload_rate_arr: [0],
	upload_prev_rate: 0,
	upload_prev_bytes: 0,
	upload_prev_time: 0,
	prev_remaining_time: 0,
	
	drawProgress: function(curIndex)
	{
		if(!curIndex) curIndex = 0;
		
		var loaded = 0;
		var total  = 0;
		for (var i = 0; i < this.files.length; i++) {
			if (!this.hasStatus(i,'overlimit')) {
				loaded += this.files[i].bytesLoaded;
				total  += this.files[i].bytesTotal;
			}
		}
		var p = loaded / total;
		
		var upload_cur_time = (new Date()).getTime();
		var upload_time_diff = upload_cur_time - this.upload_prev_time;
		var upload_bytes_diff = loaded - this.upload_prev_bytes;
		var update_rate = 3000;
		var elapsed_time = this.files[this.cur_file_pos] ? upload_cur_time - this.files[this.cur_file_pos].start_time : 0;
		
		//only update the rate every 2 seconds
		if( upload_time_diff > update_rate || this.upload_prev_rate <= 0 )
		{
			//If elapsed download time is less than a few update sessions, use previous upload rate info for calculations
			if( elapsed_time < update_rate * 2 && this.prev_rate_data )
			{
				this.upload_prev_rate = this.prev_rate_data.avg_rate;
				// debug('use previous data: ' + this.upload_prev_rate);
			}
			else
			{
				this.upload_prev_rate = upload_bytes_diff / (upload_time_diff/1000);
				// debug('use real data: ' + this.upload_prev_rate);
			}
			
			// debug(Utility.formatBytes(loaded) + ' - ' + Utility.formatBytes(this.upload_prev_bytes) + ' = ' + Utility.formatBytes(upload_bytes_diff) + ' in ' + Utility.formatSeconds((upload_time_diff/1000)));
			// debug('Bytes diff: ' + Utility.formatBytes(upload_bytes_diff) + ' Time diff: ' + Utility.formatSeconds(upload_time_diff) + ' Rate: ' + Utility.formatBytes(this.upload_prev_rate) + ' ('+this.upload_prev_rate+')');
			this.upload_prev_time = upload_cur_time;
			this.prev_remaining_time = Math.round( (total - loaded) / this.upload_prev_rate );
		}
		
		//Info
		var info = '', percent = '';
		switch(this.upload_status)
		{
			case 'saving':
				info = "Saving video info..."
			break;
			case 'adding':
				info = 'Adding video...'
			break;
			case 'finishing':
				info = 'Finishing upload...';
			break;
			case 'completed':
				info = 'Verifying upload...';
			break;
			default:
				info = Utility.formatBytes(loaded) + ' of ' + Utility.formatBytes(total) + ' (' +  Utility.formatBytes(this.upload_prev_rate) + '/s)';
				if( this.upload_prev_rate > 0 ) info += ' ' + Utility.getSimpleTime(this.prev_remaining_time) + ' remaining';
				
				percent = Math.floor(p * 100) + '%';
		}
		//If more than one file, add description of which file it is
		if( this.files.length > 1) info = info.split('...').join('') + ' ' + (curIndex+1) + ' of ' + this.files.length + '...' ;
		$('progress_info').innerHTML = info;
		
		//percentage area
		$('percent_complete').innerHTML = percent;
		
		//bar area
		$('progress_bar_area').setWidth($(document.body).getWidth() - $('progress_bar_controls').getWidth() - 30);
		
		var w = p * $('progress_bar_area').getWidth() - 2;
		w = w < 0 ? 0 : w;
		
		$('progress_meter').setWidth(w);
		
		if( upload_time_diff > update_rate ) this.upload_prev_bytes = loaded;
	},
	
	setupUploadScheduleListeners: function()
	{
		upload_schedule.addEvent('upload_schedule_set', this.updateUI.bind(this));
		upload_schedule.addEvent('upload_schedule_removed', this.updateUI.bind(this));
	}
});

/**
 * FILE DRAG & DROP
 */

function onDragEnter(e) { e.preventDefault(); }
function onDragOver(e) { e.preventDefault(); }
function onDragLeave(e) { e.preventDefault(); }
function onDragEnd(e) { }      
function onDrop(e) {
	var fileList = event.dataTransfer.getData("application/x-vnd.adobe.air.file-list");
	if (fileList) {
		for (var i = 0; i < fileList.length; i++) {
			file_manager.addFile(fileList[i]);
		}
		
		redrawWindow();
	}
}

var default_dir = false;
var current_file = false;

/**
 * Opens file dialog box
 */
function browseFile() {
	// Grab the default directory
	var stored_path = Utility.getLocalStore('default_file_dir');
	if (!stored_path) {
		default_dir = air.File.documentsDirectory;
	} else {
		var file = new air.File();
		file.nativePath = stored_path;
		default_dir = file;
	}
	
	var file_filters = new air.FileFilter("Videos", "*." + video_extensions.join(";*."))
	default_dir.browseForOpenMultiple("Open", [file_filters]);
	default_dir.addEventListener(air.FileListEvent.SELECT_MULTIPLE, openFile);
}

function openFile(event) {
	try {
		for ( var i = 0; i < event.files.length; i++) {
			file_manager.addFile(event.files[i]);
			
			// Store the directory as their new default 'Add' location
			if (i == 0) {
				var path = event.files[i].nativePath.split('/');
				path.pop();
				var tmp = path.join('/');
				Utility.setLocalStore('default_file_dir', path.join('/'));
			}
		}
	} 
	catch(error) {
		//ioErrorHandler();
	}
	
	default_dir.removeEventListener(air.Event.SELECT, openFile); 
}