/**
 * Upload Delay Manager
 */
var UploadDelayManager = new Class({
	Implements: [Options, Events],
	options: {
		updateTimeInterval: 30000,	//every 30 seconds
		scheduleText: 'Schedule&nbsp;Upload',
		editText: 'Edit&nbsp;Schedule'
	},
	
	//Assets
	element: undefined,
	edit_time: undefined,
	countdown: undefined,
	toggle_button: undefined,
	form: undefined,
	skinned_dd: undefined,
	
	//Vars
	startTime: undefined,
	state: 'empty',
	updateTimeTimer: null,
	isDisabled: false,
	
	initialize: function(element, options)
	{
		this.setOptions(options);
		
		this.element = $(element);
		this.edit_time = this.element.getElement('.edit_time');
		this.countdown = this.element.getElement('.countdown');
		this.toggle_button = this.element.getElement('.clock-icon');
		this.form = this.element.getElement('form');
		
		var startHour = eval( this._getTimeFromLocalStore().startHour );
		if( startHour ) {
			this.form.getElement('input.field').set('value', this._get12HrTime( startHour ));
			this.form.getElement('select').set('value', startHour > 12 ? "PM" : "AM");
		}
		
		skinned_dd = new Dropdown( this.form.getElement('select'), {
			width: 45
		});
		
		this._addEvents();
	},
	
	//METHODS
	_addEvents: function()
	{
		this.toggle_button.addEvent('click', function(e){
			if(this.isDisabled) return false;
			
			this.toggleState();
			e.preventDefault();
		}.bind(this));
		
		var submitFnc = function(e){
			if(this.isDisabled) return false;
			
			this.setTime(
				this.form.getElement('input.field').get('value'),
				this.form.getElement('select').get('value') == "PM"
			);
			
			e.preventDefault();
		};
		
		this.form.addEvent('submit', submitFnc.bind(this));
		this.form.getElement('input[type=button]').addEvent('click', submitFnc.bind(this));
		
		this.countdown.getElement('a.edit').addEvent('click', function(e){
			if(this.isDisabled) return false;
			
			this.showEditState();
			e.preventDefault();
		}.bind(this));
		
		this.countdown.getElement('a.remove').addEvent('click', function(e){
			if(this.isDisabled) return false;
			
			this.cancelTime();
			e.preventDefault();
		}.bind(this));
	},
	
	/**
	 * Saves last set start time as default
	 */
	_saveTimeToLocalStore: function(startHour)
	{
		if(!startHour) return false;
		
		var data = {
			startHour: startHour
		}
		
		Utility.setLocalStore('vimeo_delay_upload', JSON.encode(data));
	},
	
	_getTimeFromLocalStore: function()
	{
		try {
			data = Utility.getLocalStore('vimeo_delay_upload');
		} catch (e) {
			data = '{}';
		}
		
		return JSON.decode(data);
	},
	
	_getHourAsDate: function( hour )
	{
		var cur_time = new Date();
		var set_time = new Date(
			cur_time.getFullYear(), 
			cur_time.getMonth(), 
			cur_time.getDate(), 
			hour
		);
		
		if( cur_time > set_time ) set_time.setDate( set_time.getDate()+1 );
		
		// debug('Upload at: ' + set_time.toLocaleString());
		return set_time;
	},
	
	_updateTime: function()
	{
		if(this.isDisabled) return false;
		
		var diff = this.startTime.getTime() - (new Date()).getTime();
		if( diff <= 0 ) {
			this._startUpload();
			return false;
		}
		
		//If less than a minute left, update more frequently
		if( diff < 60000 ) {
			this.options.updateTimeInterval = 5000;
			$clear(this.updateTimeTimer);
			this.updateTimeTimer = this._updateTime.periodical( this.options.updateTimeInterval, this );
		}
		
		var time_left_el = this.countdown.getElement('span.time_left');
		time_left_el.empty();
		
		time_left_el.set('html', Utility.getSimpleTime( diff/1000 ).toLowerCase());
		
		// debug('_updateTime. remaining: ' + diff + 'mils, ' + Utility.getSimpleTime( diff/1000 ) + ' interval: ' + this.options.updateTimeInterval);
	},
	
	_startUpload: function()
	{
		if(this.isDisabled) return false;
		
		this.cancelTime();
		file_manager.startUpload();
	},
	
	_get24HrTime: function(hour, isPM)
	{
		return isPM && hour <= 12 ? hour + 12 : hour;
	},
	
	_get12HrTime: function(hour)
	{
		return hour > 12 ? hour - 12 : hour;
	},
	
	//API
	toggleState: function()
	{
		if(this.isDisabled) return false;
		
		if( this.state == 'empty' && !this.startTime )
		{
			this.showEditState();
		}
		else if( this.state == 'empty' && this.startTime)
		{
			this.showCountdownState();
		}
		else
		{
			this.showEmptyState();
		}
	},
	
	showEmptyState: function()
	{
		this.state = 'empty';
		
		if( this.startTime ) this.toggle_button.set('html', this.options.editText);
		else this.toggle_button.set('html', this.options.scheduleText);
		
		this.element.removeClass('countdown');
		this.element.removeClass('edit');
	},
	
	showEditState: function()
	{
		this.state = 'edit';
		
		this.element.removeClass('countdown');
		this.element.addClass('edit');
	},
	
	showCountdownState: function()
	{
		this.state = 'countdown';
		
		this.toggle_button.set('html', this.options.editText);
		
		this.element.removeClass('edit');
		this.element.addClass('countdown');
	},
	
	setTime: function(startHour, isPM)
	{
		this.cancelTime();
		
		this.startTime = this._getHourAsDate( this._get24HrTime(eval(startHour), isPM) );
		
		this._updateTime();
		this.updateTimeTimer = this._updateTime.periodical( this.options.updateTimeInterval, this );
		
		this.showCountdownState();
		this._saveTimeToLocalStore( this._get24HrTime(eval(startHour), isPM)) ;
	},
	
	cancelTime: function()
	{
		this.startTime = null;
		$clear(this.updateTimeTimer);
		this.showEmptyState();
	},
	
	enable: function()
	{
		if(!this.isDisabled) return false;
		this.isDisabled = false;
		this.element.removeClass('disabled');
	},
	
	disable: function()
	{
		if(this.isDisabled) return false;
		this.isDisabled = true;
		this.cancelTime();
		this.element.addClass('disabled');
	},
	
	show: function()
	{
		this.element.show();
	},
	
	hide: function()
	{
		this.element.hide();
	}
});