var Uploader = new Class({
	Implements: Options,
	options: {
		uid: false,
		chunk_size: 1024 * 1024 * 5,
		concurrent_connections: 2,
		onProgess: false,
		onComplete: false,
		onNearComplete: false,
		onError: false,
		onLock: false,
		onUnlock: false
	},
	
	file: false,
	file_cur_position: 0,
	file_contents: false,	//ByteArray of the file
	file_stream: false,	//FileStream async open file
	manifest: [], // list of all the chunks
	
	loaded: false,
	num_chunks: false,
	cur_chunk: 0,			//Current chunk that needs to be uploaded
	chunk_queue : [],	//Array of currently uploading chunks
	chunk_size : false,
	
	upload_ticket_id: false,
	upload_server_url: false,
	
	is_uploading: false,
	total_bytes_loading: 0, // total number of bytes being loaded. (when multiple chunks being uploaded at once)
	is_paused: false,
	
	num_failed_attempts: 0,
	total_allowed_attempts: 3,
	
	initialize: function(f, options) {
		this.setOptions(options);
		
		this.file = f;
		this.file_contents = false;
		
		//if the file size is less than a chunk size, divide the chunk size by the number of connections
		this.chunk_size = this.options.chunk_size * this.options.concurrent_connections > this.file.size ? this.file.size / this.options.concurrent_connections : this.options.chunk_size;
		this.num_chunks = this.getNumChunks();
		
		// debug('[uploader] filesize:' + Utility.formatBytes(this.file.size) + ' chunksize:' + this.options.chunk_size + ' numchunks:' + this.num_chunks);
		// debug("Concurrent Connections: " + this.concurrent_connections, "Number Chunks: " + this.getNumChunks() + ' Chunk size: ' + Utility.formatBytes(this.chunk_size));
		
		// init the manifest
		for (var i = 0; i < this.num_chunks; i++) {
			var s = this.getChunkSize(i);
			this.manifest.push({
				hash: '',
				stream: false,
				file: false,
				file_contents: false,
				has_tmp_file: false,
				size: s,
				start_pos: i * this.chunk_size,
				end_pos: (s - 1) + (i * this.chunk_size),
				
				is_uploading: false,
				bytesLoaded: false,
				uploaded: false,
				progress: false
			});
		}
		
		// update manifest to see if files are uploaded already
		var stored_data = this.getStoredUploadData();
		if ( stored_data ) {
			this.upload_ticket_id  = stored_data.upload_ticket_id;
			this.upload_server_url = stored_data.upload_server_url
			
			// debug('stored_data', stored_data);
			
			for (var i = 0; i < stored_data.manifest.length; i++) 
			{
				if (stored_data.manifest[i].hash != '' && this.manifest[stored_data.manifest[i].id] ) // it NEEDS a hash
				{ 
					this.manifest[stored_data.manifest[i].id].uploaded = true;
					this.manifest[stored_data.manifest[i].id].hash = stored_data.manifest[i].hash;
				}
			}
		}
	},
	
	getChunkSize: function(i) 
	{
		var s = this.file.size - (i * this.chunk_size);
		return s < this.chunk_size ? Math.abs(s) : this.chunk_size;
	},
			
	getNumChunks: function() 
	{
		return Math.ceil(this.file.size / this.chunk_size);
	},
	
	getChunkIndexByHash: function(hash) 
	{
		for (var i = 0; i < this.manifest.length; i++) {
			if ( this.manifest[i].hash == hash ) return i;
		}
		
		return false;
	},
	
	storeTempChunkFile: function(i) 
	{
		if (i < this.getNumChunks() && !this.manifest[i].has_tmp_file) {
			this.manifest[i].has_tmp_file = true;
			//this.manifest[i].file_contents.compress(air.CompressionAlgorithm.DEFLATE); // COMPRESS!
			
			try {
				if (!this.manifest[i].file_contents) {
					this.manifest[i].file_contents = new air.ByteArray();
					
					// Move the pointer to the correct part of the file
					this.file_stream.position = this.manifest[i].start_pos;
					
					this.file_stream.readBytes(this.manifest[i].file_contents, 0, this.manifest[i].size);
	
					// hash that chunk
					this.manifest[i].hash = Utility.getUniqueId(i);//Utility.md5(this.manifest[i].file_contents);
				}
			} catch(e) {
				error({ errorID: -10, text: e });
			}
			
			try {
				// save the chunk in the tmp folder
				var file_chunk = air.File.applicationStorageDirectory.resolvePath(this.manifest[i].hash + ".data");
				
				if (!file_chunk.exists) {
					var fs = new air.FileStream();
					fs.open(file_chunk, air.FileMode.WRITE);
					fs.writeBytes(this.manifest[i].file_contents);
					fs.close();
				}
				
				// Open the saved chunk
				this.manifest[i].file = new air.File(file_chunk.nativePath);
			}
			catch(e) {
				error({ errorID: -8, text: e });
			}
		}
	},
	
	removeTempChunkFile: function(i) 
	{
		if (this.manifest[i].has_tmp_file) {
			this.manifest[i].has_tmp_file = false;
			var file_chunk = air.File.applicationStorageDirectory.resolvePath(this.manifest[i].hash + ".data");
			file_chunk.deleteFileAsync();
		}
	},
	
	getStoredUploadData: function() 
	{
		return JSON.decode(Utility.getLocalStore('FileUpload' + this.file.nativePath));
	},
	
	clearTempFiles: function() 
	{
		try {
			var dir = air.File.applicationStorageDirectory;
			var list = dir.getDirectoryListing();
			
			for (var i = 0; i < list.length; i++) {
				if (list[i].nativePath.indexOf('data') != -1) {
					var f = new air.File(list[i].nativePath);
					f.deleteFile();
				}
			}
		} catch(e) {
			error({
				errorID: -9,
				text: e
			});
		}
	},
	
	updateStoredUploadData: function() {
		var data = {
			upload_ticket_id: this.upload_ticket_id,
			upload_server_url: this.upload_server_url,
			manifest: []
		}
		
		for (var i = 0; i < this.manifest.length; i++) {
			if (this.manifest[i].uploaded)
				data.manifest.push({
					id: i,
					hash: this.manifest[i].hash
				});
		}
		Utility.setLocalStore('FileUpload' + this.file.nativePath, JSON.encode(data));
	},
	
	clearStoredUploadData: function() {
		Utility.setLocalStore('FileUpload' + this.file.nativePath, '');
	},
	
	isChunkUploaded: function(i) {
		return this.manifest[i].uploaded;
	},
	
	/**
	 * Fetches a new upload ticket 
	 */
	getUploadTicket: function() {
		var tid = false;
		
		// Check if this file was uploading already
		if (tid = Utility.getLocalStore('uplod_ticket_id' + this.file.nativePath)) {
			this.upload_ticket_id  = tid;
			this.upload_server_url = Utility.getLocalStore('upload_server_url' + this.file.nativePath);
			this.upload();
		}
		// Nevermind, just upload but check the ticket
		else { 
			vimeo.getUploadTicket(this.uploadOnGetTicket.bind(this));
		}
	},
	
	uploadOnGetTicket: function(json) {
		this.upload(json, true);
	},
	
	resume: function() {
		if (this.is_paused) {
			this.is_paused = false;
			this.chunk_queue.each(function(current){
				this.uploadChunk(current);
			}.bind(this));
		}
	},
	
	pause: function() {
		if (!this.is_paused) {
			this.chunk_queue.each(function(current){
				this.resetChunk(current);
			}.bind(this));
			
			this.is_paused = true;
		}
	},
	
	cancel: function() {
		this.pause();
		
		for (var i = 0; i < this.manifest.length; i++) {
			this.manifest[i].file_contents = false;
			this.resetChunk(i);
		}
		
		this.cur_chunk 			= 0;
		this.chunk_queue		= [];
		this.upload_ticket_id 	= false;
		this.is_paused 			= false;
		this.upload_server_url 	= false;
		
		if (this.file_stream) 
			this.file_stream.close(); // close the stream
		
		this.clearStoredUploadData(); // no longer resumable once canceled
	},
	
	restartUpload: function() {
		if (network_monitor_obj.isAvailable()) {
			VimeoLogger.log('upload_restart', {});
				
			this.cancel();
			this.upload();
		}
	},
	
	retryChunk: function() {
		if (network_monitor_obj.isAvailable()) {
			this.pause();
			this.resume();
		}
	},
	
	/**
	 * Resets the specificed chunk
	 */ 
	resetChunk: function(i) {
		if( this.manifest[i] )
		{
			if (this.manifest[i].file) {
				this.manifest[i].file.cancel();
				this.removeFileListeners(this.manifest[i].file, i);
			}
			
			this.removeTempChunkFile(i);
			this.manifest[i].file 			= false;
			this.manifest[i].uploaded 		= false;
			this.manifest[i].is_uploading 	= false;
			this.manifest[i].progress 		= false;
			this.manifest[i].bytesLoaded 	= 0;
			this.manifest[i].has_tmp_file 	= false;
		}
		else
		{
			error({
				errorID: -12,
				text: 'Manifest item not found when resetting chunk'
			});
		}
	},
	
	getTotalBytesLoaded: function() {
		var total = 0;
		for (var i = 0; i < this.manifest.length; i++) {
			if (this.manifest[i].uploaded) {
				total += this.manifest[i].size;
			}
			
			if (this.manifest[i].is_uploading) {
				total += this.manifest[i].bytesLoaded;
			}
		}
		return total;
	},
			
	openFile: function() {
		// debug('openFile');
		this.file_stream = new air.FileStream();
		this.file_stream.open(this.file, air.FileMode.READ);

		this.uploadCurChunk();
	},
	
	//"generated_in":"0.0587","stat":"ok","ticket":{"endpoint":"http:\/\/ec2-174-129-136-200.compute-1.amazonaws.com\/upload_multi?ticket_id=1527d493a30590a0ede11b6ece715a00","ticket_id":"1527d493a30590a0ede11b6ece715a00"}}
	upload: function(json,no_ticket_check) {
		debug('upload', json, no_ticket_check, '-------------------');
		if (json) {
			if (!Utility.verifyJSONResponse(json)) {
				error(json);
				return false;
			}
			else {
				this.upload_ticket_id = json.ticket.id;
				this.upload_server_url = json.ticket.endpoint;
			}
		}
				
		if (!this.upload_ticket_id) {
			this.getUploadTicket();
			return false;
		}
		
		if (this.is_paused) {
			this.resume();
			return false;
		}
		
		VimeoLogger.log('upload_video_start', {
			size: this.file.size,
			upload_ticket_id: this.upload_ticket_id,
			upload_server_url: this.upload_server_url
		});
		
		debug('[uploading file:' + this.file.nativePath + ' ticket:' + this.upload_ticket_id + ' server:' + this.upload_server_url + ']')		
		this.openFile();
	},
	
	onCheckTicketResult: function(json) {
		if(!Utility.verifyJSONResponse(json)) { // start upload over
			this.restartUpload();
		} 
		else {
			this.openFile();
		}
	},
	
	//Checks to see if the chunk queue is full, meaning the maximum number of concurrent connections are open
	checkChunkQueue: function()
	{
		// debug('checking queue, length: ' + this.chunk_queue.length + ' connections: ' + this.concurrent_connections)
		if( this.chunk_queue.length < this.options.concurrent_connections && this.cur_chunk < this.num_chunks )
		{
			this.cur_chunk++;
			this.uploadCurChunk();
		}
	},
	
	uploadCurChunk: function() {
		// debug('uploadCurChunk');
		this.chunk_queue.push(this.cur_chunk);
		this.uploadChunk(this.cur_chunk);
	},
	
	uploadChunk: function(chunk_id) {
		// debug('uploadChunk: ' + chunk_id);
		// Need to check if the chunk has already been uploaded
		if (this.isChunkUploaded(chunk_id)) {
			this.onUploadComplete(false, chunk_id);
			return false;
		}
		
		// make sure there is a tmp file that we can upload
		this.storeTempChunkFile(chunk_id);  
		
		// TODO: Log how long it takes on average to complete a Video depending on number of chunks prepped at a time. Find average.
		if (this.options.onLock) 
			this.options.onLock();
			
		if (chunk_id % 1 == 0) {
			for ( var i = 1; i < 1; i++) {
				// air.trace('prepping chunk: ' + (chunk_id + i))
				this.storeTempChunkFile(chunk_id + i);
			}
		}
		
		this.is_uploading = true;
		this.manifest[chunk_id].is_uploading = true;
		if (this.options.onUnlock) 
			this.options.onUnlock();
		
		// Prep the upload
		var urlVars = new air.URLVariables();
		urlVars.chunk_id = this.manifest[chunk_id].hash;
		
		var request = new air.URLRequest(this.upload_server_url);
		request.useCache = false;
		request.method = 'POST';		
		request.data = urlVars;
		
		this.addFileListeners(this.manifest[chunk_id].file, chunk_id);
		this.manifest[chunk_id].file.upload(request, 'file_data'); // UPLOAD!!!
	},
	
	onProgress: function(e, uid) {
		try {
			
			if (this.options.onUnlock) 
				this.options.onUnlock();
			
			if (e) {
				// debug('onProgress ' + uid + ' bytesLoaded ' + e.bytesLoaded);
				var chunk_id = uid;
				this.total_bytes_loading = e.bytesLoaded;	
				this.manifest[chunk_id].bytesLoaded = e.bytesLoaded;
			}
			
			if (this.options.onProgress) 
				this.options.onProgress(this.options.uid, {
					bytesLoaded: this.getTotalBytesLoaded(),
					bytesTotal: this.file.size
				});
		} 
		catch(e) { // file probably was moved
			error({ 
				errorID: -5,
				text: e
			});
		}
	},
	
	/**
	 * This is called whenever a chunk finishes
	 * @param data - is usually a DataEvent. if this method is manually called then you must pass the chunk_id
	 */
	onUploadComplete: function(event, uid) {
		// debug('onUploadComplete chunk ' + uid + ' size ' + event.target.size, event, '------------------------');
		try {
			this.num_failed_attempts = 0; // Reset failed attempts!
		
			var chunk_id = uid;
		
			if (this.options.onLock) 
				this.options.onLock();
		
			if (event) {
				this.removeFileListeners(this.manifest[chunk_id].file, chunk_id);
			
				if (event.data != this.manifest[chunk_id].hash) { // ERROR - chunk isnt being uploaded
					debug('chunk failed uploading event: ' + event.data + ' hash: ' + this.manifest[chunk_id].hash);
					this.uploadCurChunk();
					return false;
				}
				
				this.manifest[chunk_id].bytesLoaded = event.target.size;
			}
			
			this.manifest[chunk_id].file_contents = false;
			this.manifest[chunk_id].uploaded = true;
			this.manifest[chunk_id].is_uploading = false;
			this.removeTempChunkFile(chunk_id);
		
			// update progress
			this.onProgress(false, chunk_id);
		
			this.updateStoredUploadData();
		
			//Removes the finished chunk from the uploading array
			this.chunk_queue.erase(chunk_id);
		
			//Upload the next chunk if there is another to upload
			// debug('this.cur_chunk: ' + this.cur_chunk + ' this.num_chunks-1 ' + (this.num_chunks-1));
			if( this.cur_chunk < this.num_chunks-1 )
			{
				// debug('upload next chunk+++++++');
				this.cur_chunk++;
				this.uploadCurChunk();
			}
		
			//Finished uploading all chunks, close the stream
			else if (this.cur_chunk >= this.num_chunks-1 && this.chunk_queue.length == 0) 
			{
				// debug('upload last chunk complete---------');
				this.file_stream.close();
				this.is_uploading = false;
			
				this.verifyChunks.delay(2000, this);
			}
		}
		catch(e)
		{
			error({ 
				errorID: -3,
				text: e
			});
		}
		
	},
	
	verifyChunks: function() {
		// debug('verifyChunks', 'ticket_id: ' + this.upload_ticket_id, '----------------');
		try
		{
			//Execute onNearComplete function
			if (this.options.onNearComplete) 
				this.options.onNearComplete(this.options.uid);
		
			vimeo.verifyChunks(this.upload_ticket_id, this.onVerifyChunks.bind(this));
		}
		catch(e)
		{
			error({ 
				errorID: -3,
				text: e
			});
		}
	},
	
	onVerifyChunks: function(data) {
		// debug('onVerifyChunks', data, '-----------------');
		try
		{
			//If there's no response, cancel the upload
			if( !data.ticket.chunks.chunk )
			{
				this.cancel();
			
				if (this.options.onComplete) 
					this.options.onComplete(this.options.uid, json);
			
				return false;
			}
			else
			{
				//converts chunk to array if there's only one returned
				var uploaded_chunks = data.ticket.chunks.chunk.length ? data.ticket.chunks.chunk : [data.ticket.chunks.chunk];
				var missing_chunks = [];
			
				var chunk_ids_match = true;
				var chunk_sizes_match = true;
			
				//Sort the array returned from the backend by ID
				uploaded_chunks.sort(function(a, b){
						return ( a.id - b.id );
				});
			
				//Check to see if there are any missing files or if any file sizes don't match
				// debug('manifest', this.manifest, 'uploaded_chunks.length ' + uploaded_chunks.length);
				uploaded_chunks.each(function(chunk, index, chunks){
					//Check ID's
					if( chunk.id )
						chunk_ids_match = ( chunk.id.toString() == this.manifest[index].hash.toString() );
				
					//Check sizes
					if( chunk.size )
						chunk_sizes_match = ( chunk.size.toString() == this.manifest[index].bytesLoaded.toString() );
				
					//Add any missing or incomplete chunks to missing_chunks array and mark for re-upload
					if( !chunk_ids_match || !chunk_sizes_match ) {
						debug('Chunk index: ' + index + ' is invalid, ids_match: ' + chunk_ids_match + ' sizes_match: ' + chunk_sizes_match);
						debug('chunk_ids_match. ' + chunk_ids_match + '. chunk.id: ' + chunk.id + ' this.manifest['+index+'].hash: ' + this.manifest[index].hash);
						debug('chunk_sizes_match. ' + chunk_sizes_match + '. chunk.size: ' + chunk.size + ' this.manifest['+index+'].bytesLoaded: ' + this.manifest[index].bytesLoaded);
						missing_chunks.push(chunk);
						this.resetChunk(index);
					}
				}.bind(this));
			
				//Check missing_chunks list. If it's greater than 0, re-upload the missing chunks
				if(missing_chunks.length > 0)
				{
					this.cur_chunk = 0;
					this.onProgress(false, 0);
					this.upload();
				}
				//If there are no missing_chunks, the chunks are verified. Add the video
				else
				{
					this.complete();
				}
			}
		}
		catch(e)
		{
			error({ 
				errorID: -3,
				text: e
			});
		}
	},
	
	complete: function() {
		// debug('complete');
		vimeo.complete(this.file.name, this.upload_ticket_id, this.onVideoComplete.bind(this));
	},
	
	onVideoComplete: function(json) {
		// debug('onVideoComplete', json, '--------------');
		try
		{
			this.clearStoredUploadData(); // no longer resumable once finished
		
			if (this.options.onComplete) 
				this.options.onComplete(this.options.uid, json);
		
			if(json)
			{
				VimeoLogger.log('upload_complete', {
					video_id: json.ticket.video_id,
					file_size: this.file.size
				});
			}
		}
		catch(e)
		{
			error({ 
				errorID: -3,
				text: e
			});
		}
	},
	
	onIoError: function(e) {
		debug('IO Error: ' + e);
		
		this.num_failed_attempts++;
		debug('Attempt #' + (this.num_failed_attempts+1));
		
		// Server is gone or upload ticket is expired. start over
		if (this.num_failed_attempts >= this.total_allowed_attempts) { 
			this.num_failed_attempts = 0;
			
			this.restartUpload.bind();
			
			//Log the error once all 3 attempts fails
			VimeoLogger.error({
				error_id: e.errorID.toInt(),
				msg: 'Uploader:IOError,attempt:' + (this.num_failed_attempts+1) + ',chunk:' + this.cur_chunk + '/' + this.num_chunks + ',' + e + ',' + this.upload_server_url + ',' + this.upload_ticket_id
			});
		}
		else 
		{ // Try again
			this.retryChunk.delay(3000, this);
		}
	},
	
	onSecurityError: function(e) {
		debug('Security Error: ' + e);
	},
	
	onHttpStatus: function(e) {
		debug('HTTP Status: ' + e);
	},
	
	onOpen: function(e) {
		//debug('Open: ' + e);
		this.checkChunkQueue();
	},
	
	addFileListeners: function(f, uid) {
		// air.trace("ADD FILE LISTENERS, uid: " + uid);
		f.addEventListener(air.ProgressEvent.PROGRESS, this.onProgress.bindWithEvent(this, uid));
		f.addEventListener(air.IOErrorEvent.IO_ERROR, this.onIoError.bind(this));
		f.addEventListener(air.SecurityErrorEvent.SECURITY_ERROR, this.onSecurityError.bind(this));
        f.addEventListener(air.HTTPStatusEvent.HTTP_STATUS, this.onHttpStatus.bind(this));
		f.addEventListener(air.Event.OPEN, this.onOpen.bind(this));
		//f.addEventListener(air.Event.COMPLETE, this.uploadStartComplete.bind(this));
		f.addEventListener(air.DataEvent.UPLOAD_COMPLETE_DATA, this.onUploadComplete.bindWithEvent(this, uid));
	},
	
	removeFileListeners: function(f, uid) {
		if (f) {
			f.removeEventListener(air.ProgressEvent.PROGRESS, this.onProgress.bindWithEvent(this, uid));
			f.removeEventListener(air.IOErrorEvent.IO_ERROR, this.onIoError.bind(this));
			f.removeEventListener(air.SecurityErrorEvent.SECURITY_ERROR, this.onSecurityError.bind(this));
	        f.removeEventListener(air.HTTPStatusEvent.HTTP_STATUS, this.onHttpStatus.bind(this));
			f.removeEventListener(air.Event.OPEN, this.onOpen.bind(this));
			//f.removeEventListener(air.Event.COMPLETE, this.uploadStartComplete.bind(this));
			f.removeEventListener(air.DataEvent.UPLOAD_COMPLETE_DATA, this.onUploadComplete.bindWithEvent(this, uid));
		}
	}
});