var VimeoAPI = new Class({
	dev: DEV,
	
	key: 	'57974ededb06767be492595089426d0a',
	secret:	false,
	
	domain: VIMEO_URL,
	frob: false,
	token: false,
	logged_in: false,
	
	user_id: false, // logged in user
	username: false, // logged in username
	user_info: { }, // user info
	user_upload_status: { },
	
	user_groups: false,
	user_channels: false,
	user_albums: false,
	
	auth_loader: false,
	
	// upload stuff
	video_file: false,
	pending_albums_manifest: {},
	
	// hooks
	call_on_complete: false,
	on_get_auth_url: false, // auth url hook
	
	// permissions
	PERM_READ: 	 'read',
	PERM_WRITE:  'write',
	PERM_DELETE: 'delete',
	
	options: {
		on_authorize: false,
		on_login: false,
		on_get_user_info: false,
		on_get_user_portrait: false
	},
	
	initialize: function(options) { 
		this.setOptions(options);
		
		this.secret = Utility.getAPISecret();
		
		// this.token = air.EncryptedLocalStore.getItem('vimeoapi_token');
		var vimeoapi_token = air.EncryptedLocalStore.getItem('vimeoapi_token');
		if (vimeoapi_token) {
			this.token = vimeoapi_token.readUTFBytes(vimeoapi_token.length);
		}
	},
	
	login: function() {
		if (!this.token) {
			this.getFrob();
		}
		else {
			// Double check token
			this.checkToken();
		}
	},
	
	logout: function() {
		this.logged_in = false;
		this.user_info.portrait = false;
		this.user_info.id 		= false;
		air.EncryptedLocalStore.removeItem('vimeoapi_token');
		this.getFrob();
	},
	
	getFrob: function() {
		this.call({
			method: 'vimeo.auth.getFrob'
		}, this.getFrobComplete);
	},
	
	// {"stat":"ok","generated_in":"0.0010","frob":"32bb06c2c1d0aecfcc516dbc8c7fcd1e"}
	getFrobComplete: function(json) {
		if (Utility.verifyJSONResponse(json)) {
			this.frob = json.frob;
			
			// need to let user authorize
			if (this.options.on_authorize) 
				this.options.on_authorize();
		}
		else {
			error(json);
		}
	},
	
	authorize: function() {
		Utility.goToURL(this.getAuthURL());
	},
	
	getAuthURL: function() {
		var url = 'http://' + this.domain + '/services/auth?api_key=' + this.key + '&perms=write&frob=' + this.frob + '&api_sig=' + this.getSignature({
			perms: 'write',
			frob: this.frob
		});
		//air.trace('Authorizing: ' + url);
		return url;
	},
	
	/**
	 * Checks the validity of the token. Returns the user associated with it.
	 * Returns the same as vimeo.auth.getToken
	 */
	checkToken: function() {
		this.call({
			auth_token: this.token,
			method: 'vimeo.auth.checkToken'
		}, this.getTokenComplete)
	},
	
	getToken: function() {
		this.call({
			frob: this.frob,
			method: 'vimeo.auth.getToken'
		}, this.getTokenComplete)
	},
	
	// {"stat":"ok","generated_in":"0.0035","auth":{"token":"b2932f5aa62266a6125e144e1c726dc6","perms":"delete","user":{"nsid":"147469","id":"147469","username":"casey","fullname":"Casey Pugh"}}}
	getTokenComplete: function(json) {
		if (Utility.verifyJSONResponse(json)) {
			this.token = json.auth.token;
			
			// Store the token 4ever
			var bytes = new air.ByteArray();
            bytes.writeUTFBytes(this.token);
			air.EncryptedLocalStore.setItem('vimeoapi_token', bytes);
			
			this.user_id  = json.auth.user.id; // user_id
			this.username = json.auth.user.username; // username
			this.logged_in = true;
			
			// We're done logging in
			if (this.options.on_login ) this.options.on_login();
		}
		else {
			error(json);
			
			this.token = '';
            air.EncryptedLocalStore.removeItem('vimeoapi_token');
			// debug('Getting new token...');
			this.getFrob();
		}
	},
	
	getUserInfo: function() {
		return this.call({
			method: 'vimeo.people.getInfo',
			auth_token: this.token
		}, this.getUserInfoComplete);
	},
	
	getUserInfoComplete: function(json) {
		$extend(this.user_info, json.person);
		
		// external hook
		if (this.options.on_get_user_info) 
			this.options.on_get_user_info();
	},
	
	getUserPortrait: function() {
		return this.call({
			method: 'vimeo.people.getPortraitURLs',
			auth_token: this.token
		}, this.getUserPortraitComplete);
	},
	
	getUserPortraitComplete: function(json) {
		this.user_info.portrait = false;
		
		if (Utility.verifyJSONResponse(json)) {
			for (var i = 0; i < json.portraits.portrait.length; i++) {
				if (json.portraits.portrait[i].height == 30) {
					this.user_info.portrait = {
						src: json.portraits.portrait[i]._content,
						width: json.portraits.portrait[i].width,
						height: json.portraits.portrait[i].height
					}
					break;
				}
			}
		}
		
		if (this.options.on_get_user_portrait) 
			this.options.on_get_user_portrait();
	},
	
	getQuota: function(fn) {
		return this.call({
			method: 'vimeo.videos.upload.getQuota',
			auth_token: this.token			
		}, fn);
	},
	
	setTitle: function(video_id, title, on_complete) {
		debug('VimeoAPI.setTitle', video_id, title);
		if (title != '' )
		{
			return this.call({
				method: 'vimeo.videos.setTitle',
				video_id: video_id,
				title: SwfUtility.utf8Encode(title),
				auth_token: this.token
			}, on_complete);
		}
		else
		{
			return null;
		}
	},
	
	setDescription: function(video_id, caption, on_complete) {
		debug('VimeoAPI.setDescription', video_id, caption);
		if (caption != '' )
		{
			return this.call({
				method: 'vimeo.videos.setDescription',
				video_id: video_id,
				description: SwfUtility.utf8Encode(caption),
				auth_token: this.token
			}, on_complete);
		}
		else
		{
			return null;
		}
	},
	
	setTags: function(video_id, tags, on_complete) {
		debug('VimeoAPI.setTags', video_id, tags);
		if (tags != '' )
		{
			return this.call({
				method: 'vimeo.videos.addTags',
				video_id: video_id,
				tags: SwfUtility.utf8Encode(tags),
				auth_token: this.token
			}, on_complete);
		}
		else
		{
			return null;
		}
	},

	setPrivacy: function(video_id, privacy, users, password, on_complete) {
		debug('VimeoAPI.setPrivacy', video_id, privacy);
		var params = {
			method: 'vimeo.videos.setPrivacy',
			video_id: video_id,
			privacy: privacy,
			users: users,
			auth_token: this.token
		};
		
		//Check for password
		if(privacy == 'password' && password != '') params.password = password;
		
		return this.call(params, on_complete);
	},
	
	setAddTo: function(video_id, items, onVideoAdded, onAddFailed)
	{
		var length = items.length, item;
		var groups = [], channels = [], albums = [];
		
		for (var i=0; i < length; i++) 
		{
			item = items[i];
			
			if( item.indexOf('Groups') >= 0 )
			{
				groups.push(item.split('Groups_').join(''));
			}
			if( item.indexOf('Channels') >= 0 )
			{
				channels.push(item.split('Channels_').join(''));
			}
			if( item.indexOf('PendingAlbums') >= 0 )
			{
				albums.push(item.split('PendingAlbums_').join(''));
			}
			else if( item.indexOf('Albums') >= 0 )
			{
				albums.push(item.split('Albums_').join(''));
			}
		};
		
		var batch_ajax = new BatchAJAX(onVideoAdded, onAddFailed);
		
		//Add Albums to batch
		if( albums.length > 0 )
		{
			albums.each(function(album_id){
				// debug('addAJAX: ' + album_id + ', ' + video_id);
				batch_ajax.addAJAX( this.addToAlbum(album_id, video_id, onAddFailed) );
			}.bind(this));
		}
		
		//Add Groups to batch
		if( groups.length > 0 )
		{
			groups.each(function(group_id){
				// debug('addAJAX: ' + group_id + ', ' + video_id);
				batch_ajax.addAJAX( this.addToGroup(group_id, video_id) );
			}.bind(this));
		}
		
		//Add Channels to batch
		if( channels.length > 0 )
		{
			channels.each(function(channel_id){
				// debug('addAJAX: ' + channel_id + ', ' + video_id);
				batch_ajax.addAJAX( this.addToChannel(channel_id, video_id) );
			}.bind(this));
		}
	},
	
	addToGroup: function(group_id, video_id)
	{
		return this.call({
			method: 'vimeo.groups.addVideo',
			group_id: group_id,
			video_id: video_id,
			auth_token: this.token,
			format: 'json'
		});
	},
	
	addToChannel: function(channel_id, video_id)
	{
		return this.call({
			method: 'vimeo.channels.addVideo',
			channel_id: channel_id,
			video_id: video_id,
			auth_token: this.token,
			format: 'json'
		});
	},
	
	addToAlbum: function(album_id, video_id, failFnc)
	{
		// attempt = attempt || 0;
		
		return this.call({
			method: 'vimeo.albums.addVideo',
			album_id: album_id,
			video_id: video_id,
			auth_token: this.token,
			format: 'json'
		}, 
		function( data ){
			debug('onAlbumAdded', data);
			
			//If returns an error because the album was not found, create it
			if( data.err )
			{
				debug('onAlbumAddedFailed Error Code: ' + data.err.code.toString());
				
				switch( data.err.code.toString() )
				{
					case "1": 		//Album not found
					case "-1": 		//General Internal Error.
						debug('Checking against pending albums manifest... :' + this.pending_albums_manifest[album_id]);
						//check to make sure this pending album hasn't already been created
						if( !this.pending_albums_manifest[album_id] )
						{
							debug('this is a new album. create it.');
							this.createAlbum( 
								video_id, 
								$('PendingAlbums_'+album_id).getNext('.title').get('text'),
								$('PendingAlbums_'+album_id).getNext('.description').get('text'),
								function(data) {
									//Add new album ID to pending albums manifest
									debug("onAlbumCreated: " + data.album[0].id + " vs " + this.pending_albums_manifest[album_id], data);
									if( !data.err ) {
										this.pending_albums_manifest[album_id] = data.album[0].id;
									}
									else {
										debug('call fail fnc');
										failFnc.call();
									}
									
								}.bind(this)
							);
						}
						else
						{
							debug('pending album: ' + album_id + ' has already been created. use this album id: ' + this.pending_albums_manifest[album_id]);
							this.addToAlbum(this.pending_albums_manifest[album_id], video_id);
						}
					break;
				}
			}
			else 
			{
				debug('onAlbumAddedSuccess');
				// if(fn) fn.call();
			}
		}.bind(this));
	},
	
	checkPendingAlbumsManifest: function(pending_id)
	{
		debug('Checking against pending albums manifest...',album_id + ' vs ' + this.pending_albums_manifest[album_id]);
		this.pending_albums_manifest.each(function(item, index){
			
		}.bind(this));
	},
	
	createAlbum: function(video_id, title, description, fn)
	{
		if( !title || title == '') title = 'Untitled';
		
		return this.call({
			method: 'vimeo.albums.create',
			video_id: video_id,
			title: SwfUtility.utf8Encode(title),
			description: SwfUtility.utf8Encode(description),
			auth_token: this.token,
			format: 'json'
		}, fn);
	},
	
	uploadVideo: function(file) {
		this.video_file = file;
		
		this.getUploadTicket();
	},
	
	getUploadTicket: function(fn) {
		return this.call({
			method: 'vimeo.videos.upload.getTicket',
			auth_token: this.token
		}, fn);
	},
	
	checkTicket: function( ticket_id, fn ) {
		this.call( {
			method: 'vimeo.videos.upload.checkTicket',
			auth_token: this.token,
			ticket_id: ticket_id },
			fn
		);
	},
	
	getGroups: function()
	{
		return this.call({
			method: 'vimeo.groups.getAddable',
			user_id: this.user_id,
			// auth_token: this.token,
			format: 'json'
		}, this.onGetGroups);
	},
	
	onGetGroups: function( json )
	{
		if(!json.groups.group){
			this.user_groups = ['You have not created any groups. <a href="http://vimeo.com/groups/create">Create one</a>.'];
			return false;
		}
		
		this.user_groups = json.groups.total > 1 ? json.groups.group : [json.groups.group];
		this.user_groups = !this.user_groups[0] || !this.user_groups[0].name ? ['You have not created any groups. <a href="http://vimeo.com/groups/create">Create one</a>.'] : this.user_groups;
	},
	
	getChannels: function()
	{
		return this.call({
			method: 'vimeo.channels.getModerated',
			user_id: this.user_id,
			// auth_token: this.token,
			format: 'json'
		}, this.onGetChannels);
	},
	
	onGetChannels: function( json )
	{
		if(!json.channels.channel){
			this.user_channels = ['You have not created a channel yet. <a href="http://vimeo.com/channels/create">Create one</a>.'];
			return false;
		}
		
		this.user_channels = json.channels.total > 1 ? json.channels.channel : [json.channels.channel];
		this.user_channels = !this.user_channels[0] || !this.user_channels[0].name ? ['You have not created a channel yet. <a href="http://vimeo.com/channels/create">Create one</a>.'] : this.user_channels;
	},
	
	getAlbums: function()
	{
		return this.call({
			method: 'vimeo.albums.getAll',
			user_id: this.user_id,
			// auth_token: this.token,
			format: 'json'
		}, this.onGetAlbums);
	},
	
	onGetAlbums: function( json )
	{
		if(!json.albums.album){
			this.user_albums = ['You do not have any public albums. Only non-private albums are currently supported.'];
			return false;
		}
		
		this.user_albums = json.albums.total > 1 ? json.albums.album : [json.albums.album];
		this.user_albums = !this.user_albums[0] || !this.user_albums[0].title ? ['You have not created any albums.'] : this.user_albums;
	},
	
	verifyChunks: function(ticket_id, fn) {
		// debug('Vimeo.verifyChunks', ticket_id, fn, '---------------');
		this.call({
			method: 'vimeo.videos.upload.verifyChunks',
			auth_token: this.token,
			ticket_id: ticket_id
		}, fn);
	},
	
	// verifyManifest: function(manifest, ticket_id, fn) {
	// 	this.call({
	// 		method: 'vimeo.videos.upload.verifyManifest',
	// 		auth_token: this.token,
	// 		ticket_id: ticket_id,
	// 		post: {
	// 			json_manifest: JSON.encode(manifest)
	// 		}
	// 	}, fn);
	// 	air.trace( "manifest: " + JSON.encode(manifest) );
	// },
	
	complete: function(filename, ticket_id, fn) {
		// this.call({
		// 	method: 'vimeo.videos.upload.complete',
		// 	auth_token: this.token,
		// 	ticket_id: ticket_id,
		// 	filename: filename
		// }, fn);
		
		/**
		 * Using javascript JSON request since there's a bug in Flash's
		 * URLLoader in AIR 1.0 where it has a forced 60 second timeout.
		 **/
		var params = {
			method: 'vimeo.videos.upload.complete',
			auth_token: this.token,
			ticket_id: ticket_id,
			filename: filename,
			format: 'json'
		};
		
		var url = 'http://' + this.domain + '/api/rest/v2?api_key=' + this.key + '&api_sig=' + this.getSignature(params);
		$each(params, function(val, key) {
			if (key != 'post' )
				url += '&' + key + '=' + escape(val);
		});
		
		var request = new Request.JSON({url: url});
		request.addEvent('success', fn);
		request.send();
	},
	
	// addVideo: function(filename, manifest, ticket_id, fn) {
	// 	this.call({
	// 		method: 'vimeo.videos.upload.confirm',
	// 		auth_token: this.token,
	// 		ticket_id: ticket_id,
	// 		filename: filename,
	// 		post: {
	// 			json_manifest: JSON.encode(manifest)
	// 		}
	// 	}, fn);
	// },
	
	addFileListeners: function(f) {
		f.addEventListener(air.ProgressEvent.PROGRESS, this.onProgress.bind(this));
		f.addEventListener(air.IOErrorEvent.IO_ERROR, this.onIoError.bind(this));
		f.addEventListener(air.SecurityErrorEvent.SECURITY_ERROR, this.onSecurityError.bind(this));
        f.addEventListener(air.HTTPStatusEvent.HTTP_STATUS, this.onHttpStatus.bind(this));
		f.addEventListener(air.Event.OPEN, this.onOpen.bind(this));
		//f.addEventListener(air.Event.COMPLETE, this.uploadStartComplete.bind(this));
		f.addEventListener(air.DataEvent.UPLOAD_COMPLETE_DATA, this.onUploadComplete.bind(this));
	},
	
	removeFileListeners: function(f) {
		f.removeEventListener(air.ProgressEvent.PROGRESS, this.onProgress.bind(this));
		f.removeEventListener(air.IOErrorEvent.IO_ERROR, this.onIoError.bind(this));
		f.removeEventListener(air.SecurityErrorEvent.SECURITY_ERROR, this.onSecurityError.bind(this));
        f.removeEventListener(air.HTTPStatusEvent.HTTP_STATUS, this.onHttpStatus.bind(this));
		f.removeEventListener(air.Event.OPEN, this.onOpen.bind(this));
		//f.removeEventListener(air.Event.COMPLETE, this.uploadStartComplete.bind(this));
		f.removeEventListener(air.DataEvent.UPLOAD_COMPLETE_DATA, this.onUploadComplete.bind(this));
	},
	
	startUpload: function(json) {
		// air.trace('UPLOADZ');
	},
	
	getSignature: function(params) {
		var keys = ['api_key'];
		$each(params, function(value, key) {
			if (key != 'post' )
				keys.push(key);
		});
		keys.sort();
		
		var base_sig = this.secret;// + 'api_key' + this.key;
		
		// Sort the params in alphabetical order!
		for (var i = 0; i < keys.length; i++) {
			$each(params, function(value, key, index){
				if (key == keys[i]) {
					base_sig += key + value;
				}
			}.bind(this));
			
			if( keys[i] == 'api_key') {
				base_sig += 'api_key' + this.key;
			}
		}
		
		return hex_md5(base_sig);
	},

	call: function(params, onComplete, oncomplete_args) {
		this.call_on_complete = onComplete ? onComplete : false;
		
		var format = {
			format: 'json'
		}
		$extend(params, format);

		var url = 'http://' + this.domain + '/api/rest/v2?api_key=' + this.key + '&api_sig=' + this.getSignature(params);
		
		$each(params, function(val, key) {
			if (key != 'post' && key!= 'silence_errors' )
				url += '&' + key + '=' + escape(val);
		});
		
		var attempts = 3;
		if (params.method == 'vimeo.auth.checkToken' || params.method == 'vimeo.videos.upload.checkTicket') {
			attempts = false;
		}
		
		if (onComplete) {
			return new AJAX(url, { 
				response_format: 'json', 
				onComplete: onComplete.bind(this), 
				oncomplete_args: oncomplete_args, 
				post: params.post, 
				allow_attempts: attempts,
				error_response_check: '"stat":"fail"',
				silence_errors: params.silence_errors
			});
		}
		else {
			return new AJAX(url, { 
				response_format: 'json', 
				post: params.post,
				allow_attempts: attempts,
				error_response_check: '"stat":"fail"',
				silence_errors: params.silence_errors
			});
		}
	}
});
VimeoAPI.implement(new Options);