window.addEvent('load', init);

var DEV	= false;
var DEBUG_MODE = false;					//
var VIMEO_URL = 'vimeo.com';			//Replaced on build

var vimeo = false; // Vimeo API object
var video_extensions = ['3g2', '3gp', '3gp2', '3gpp', 'asf', 'avi', 'd2v', 'd3v', 'dat', 'dif', 'divx', 'dv', 'dvr-ms', 'dvx', 'm1v', 'm2t', 'm2ts', 'm2v', 'm4v', 'mkv', 'moov', 'mov', 'movie', 'mp4', 'mpe', 'mpeg', 'mpg', 'mpv2', 'mts', 'ogm', 'qt', 'ts', 'vid', 'vob', 'wm', 'wmv', 'xvid', 'yuv'];
var file_manager = false; // FileManager.js
var network_monitor_obj = false; // NetworkMonitor.js
var is_user_idle = false;
var upload_schedule = null;

function debug(str)
{
	// if( DEBUG_MODE ) air.trace(str + '\n');
	if( DEBUG_MODE )
	{
		for(var i=0; i<arguments.length; i++)
		{
			air.trace(arguments[i]);
		}
	}
}

function init()
{
	//Biggest hack ever. Don't ask. Reroutes traces to Introspector Console log
	//@author joseph.schmitt
	if( air.Introspector.Console.log && DEBUG_MODE ) {
		air.trace = air.Introspector.Console.log;
	}
	
	debug('/////////////////////////////////////////');
	debug('////////////// VIMEO DUPLO //////////////');
	debug('//////////// '+VIMEO_URL+' ////////////');
	debug('/////////////////////////////////////////');
		
	// Setup App listeners
	redrawWindow();
	redrawWindow.periodical(500, false, 'test');
	nativeWindow.addEventListener(air.NativeWindowBoundsEvent.RESIZE, redrawWindow);
	
	air.NativeApplication.nativeApplication.idleThreshold = 120; // 2 minutes
	air.NativeApplication.nativeApplication.addEventListener(air.Event.USER_IDLE, function() {
		is_user_idle = true;
		// debug('USER IDLE');
	}); 
	air.NativeApplication.nativeApplication.addEventListener(air.Event.USER_PRESENT, function() {
		is_user_idle = false;
		// debug('USER PRESENT');
	});
	
	//error catcher
	window.htmlLoader.addEventListener("uncaughtScriptException", onUncaughtScriptError); 
	
	file_manager = new FileManager();
	
	// upload_schedule = new UploadScheduleManager($('upload_schedule_area'));
	// file_manager.setupUploadScheduleListeners();
	
	// start monitoring the user's connection
	network_monitor_obj = new NetworkMonitor();
	network_monitor_obj.start();
	
	// Start the VimeoAPI object and pass in event hooks
	vimeo = new VimeoAPI({
		on_authorize: api_onAuthorize,
		on_login: api_onLogin
	});

	// Preload images
	new Asset.images('/images/upload_arrow_hover.gif');
	
	vimeo.login(); // login to api
	loading_lightbox.show(); // show loading animation
	
	VimeoLogger.log('load'); // log the load
	
	// Refresh the quota every 5 minutes
	api_updateUploadStatus.periodical(1000 * 60 * 5);
	
	// On window closing event
	nativeWindow.addEventListener(air.Event.CLOSING, onWindowClosing);
}

function onWindowClosing(e)
{
	if( file_manager.is_uploading && !file_manager.is_paused ) 
	{
		e.preventDefault();
		quit_lightbox.show();
	}
	else
	{
		doWindowClose();
	}
}

function doWindowClose()
{
	if( file_manager.is_uploading && !file_manager.is_paused )
	{
		file_manager.pauseUpload();
	}
	
	window.nativeWindow.close();
}

function onUncaughtScriptError(e)
{
	e.preventDefault(); 
	
	var message = "exceptionValue:" + e.exceptionValue + "\n"; 
	for (var i = 0; i < e.stackTrace.length; i++) { 
		message += "sourceURL:" + e.stackTrace[i].sourceURL +"\n"; 
		message += "line:" + e.stackTrace[i].line +"\n"; 
		message += "function:" + e.stackTrace[i].functionName + "\n"; 
	} 
	
	error({
		errorID: -3,
		text: message
	});
}

function show_message(txt)
{
	$('error_msg').innerHTML = txt;
	error_lightbox.show();
}

function error(data)
{
	loading_lightbox.hide();
	
	var error_id = 0;
	var msg = '';
	
	if (data === undefined || data === null) {
		error_id = -2;
	}
	else if (data.errorID) {
		error_id = data.errorID.toInt();
		msg = data.text;
	}
	else if (data.err) {
		error_id = data.err.code.toInt();
		msg = data.err.msg;
	}
	else if (data.text) { // if just text, then show custom text
		error_id = -1;
		msg = data.text;
	}
	
	$('error_ok_btn').show();
	$('reconnect_btn').hide();
	$('error_cancel_btn').hide();
	$('error_retry_btn').hide();
	$('error_skip_btn').hide();
	$('error_wait_btn').hide();
	
	if (error_id != -4 && error_id != -6) {
		debug('error data', data);
		VimeoLogger.error({
			error_id: error_id,
			msg: msg
		});
	}
	
	// Silence specific errors on the client side (we're still logging)
	if (error_id == -2) {
		return false;
	}
	
	debug('error()', 'error id: ' + error_id, data, '---------------------------');
	
	switch (error_id) 
	{
		//Custom errors
		case -1: // Acceptable video formats
			$('error_msg').innerHTML = data.text;
			break;
			
		case -2: // Null error... not sure what this exactly is every time. Silenced for now
			if (file_manager) file_manager.pauseUpload();
				
			$('error_msg').innerHTML = 'Service is temporarily unavailable. Please try again later.';
			
			if (msg) $('error_msg').innerHTML += ' (' + msg + ')';
			break;
		
		case -3: // Some sort of JS error caught by the AJAX onComplete method
		case -7: // AJAX IOerrors (usually a 2032 stream error?)
		case -8: // TMP file saving error
		case -9: // Clearing tmp files errorz
		case -10: // Error reading the video file
			//Button visibility
			$('error_ok_btn').hide();
			
			$('error_cancel_btn').show();
			$('error_retry_btn').show();
			$('error_skip_btn').show();
			
			//Message text
			$('error_msg').innerHTML = "Uh oh! The Vimeo Uploader has encountered an error. Would you like to try again?";
			
			if( file_manager ) 
			{
				file_manager.pauseUpload();
				
				switch(file_manager.upload_status)
				{
					case 'uploading':
					case 'paused':
					case 'completed':
						file_manager.unlockUploadControls();
					break;
				}
			}
			break;
			
		case -4: // Non plus overlimit message
			$('error_msg').innerHTML = "You have exceeded your weekly disk space.";
			break;
			
		case -5: // they moved the file
			$('error_msg').innerHTML = "Whoops! Looks like the video you're uploading was either moved, renamed or deleted.";
			if (file_manager) file_manager.cancelUpload();
			break;
			
		case -6: // Connection no longer available
			$('error_ok_btn').hide();
			$('error_msg').innerHTML = "Uh oh! Your connection has been lost. Please verify that you are connected to the internet.";
			break;
		
		case -11: // Controls unresponsive
			//Button visibility
			$('error_ok_btn').hide();
			
			$('error_cancel_btn').show();
			$('error_wait_btn').show();
			
			if( file_manager ) 
			{
				file_manager.pauseUpload();
				// file_manager.unlockUploadControls();
			}

			$('error_msg').innerHTML = "Uh oh! It seems like the application has become unresponsive. Would you like to cancel your upload?";
			break;
		
		case -12: // Manifest item not found when resetting chunk
			//Button visibility
			$('error_ok_btn').hide();
			
			$('error_cancel_btn').show();
			$('error_wait_btn').show();
			
			$('error_msg').innerHTML = "Uh oh! Something went wrong. Would you like to try again?";
			break;
		
		//Flash errors
		case 2032: // [Event] bad ajax call - Stream Error
			if (file_manager) file_manager.cancelUpload();
			$('error_msg').innerHTML = "An error has occurred. Please check your connection.";
			$('error_ok_btn').hide();
			$('reconnect_btn').show();
			break;
		
		case 98: // [API] Login failed / invalid auth token 
			logout();
			return false;
			break;
			
		case 105: // [API] Service currently unavailable
			$('error_msg').innerHTML = "Service is temporarily unavailable. Please try reconnecting in a minute.";
			$('error_ok_btn').hide();
			$('reconnect_btn').show();
			break;
			
		case 108: // [API] invalid frob. most likely they didnt authorize the app.
			$('error_msg').innerHTML = "We were unable to start the Desktop Uploader. When your default browser opens after clicking the <b>Authorize</b> button, make sure you click <b>Yes, authorize this app!</b>. Then you can click the <b>Start</b> button to proceed.";
			break;
			
		/*case 701: // [API] cannot upload
		
			break;*/
			
		case 703: // [API] This user has exceeded the daily amount of uploads
			$('error_msg').innerHTML = "Sorry, you've hit the maximum number of videos you can upload today. You can either wait until tomorrow to upload more, or <span id=\"error_plus_click\" class=\"faux_link\">upgrade to Plus</span> and continue uploading right now.";
			$('error_plus_click').addEvent('click', function() {
				Utility.goToURL('http://www.vimeo.com/plus');
				VimeoLogger.log('plus_promo_click');
			});
			break;
			
		default:
			$('error_msg').innerHTML = '(Error #' + error_id + ') ' + msg;
			$('error_ok_btn').hide();
			$('reconnect_btn').show();
			break;
	}
	
	error_lightbox.show();
}

function show_upload_rules()
{
	var has_shown = Utility.getLocalStore('show_upload_rules');
	if (has_shown != 1) {
    	firsttime_lightbox.show();
		Utility.setLocalStore('show_upload_rules', 1);
	}
}

function reset_upload_rules()
{
	Utility.setLocalStore('show_upload_rules', 0);
}

function logout()
{
	reset_upload_rules();
	
	// cancel uploads
	file_manager.cancelUpload();
	file_manager.resetDiskSpace();
	
	// remove all files
	file_manager.selectAllFiles();
	file_manager.removeSelectedFiles();
	
	// Update UI
	$('login_area').show();
	$('authorize_area').show();
	$('meat').hide();
	$('footer').hide();
	$('user_meta').hide();
	$('header_controls').hide();
	$('upload_status_area').setStyle('visibility', 'hidden');
	$('start_area').hide();
	
	VimeoLogger.log('logged_out');
	
	vimeo.logout();
	
	redrawWindow();
}

function showProgressBar()
{
	$('upload_status_area').hide();
	$('progress_area').show();
	redrawWindow();
}

function authorize()
{
	vimeo.authorize();
	$('authorize_area').hide();
	$('start_area').show();
	redrawWindow();
}

function startUploader()
{
	loading_lightbox.show();
	$('start_area').hide(); 
	vimeo.getToken();
}

function api_onAuthorize()
{
	$('authorize_area').show();
	$('start_area').hide();
	
	$('authorize_url_txt').value = vimeo.getAuthURL();
	
	loading_lightbox.hide();
	
	redrawWindow();
}

function api_onLogin()
{	
	// debug('Logged in -- fetching user info');
	
	// Logged in -- now load in all the user info
	var batch_ajax = new BatchAJAX(startUploaderFinished);

	if (!vimeo.user_info.id)
		batch_ajax.addAJAX(vimeo.getUserInfo());
	
	if (!vimeo.user_info.portrait)
		batch_ajax.addAJAX(vimeo.getUserPortrait());
	
	if (!file_manager.disk_usage)
		batch_ajax.addAJAX(api_updateUploadStatus());
	
	if( !file_manager.max_file_size)
		batch_ajax.addAJAX(api_getMaxFileSize());
	
	if( !vimeo.user_groups )
		batch_ajax.addAJAX(vimeo.getGroups());

	if( !vimeo.user_channels )
		batch_ajax.addAJAX(vimeo.getChannels());
		
	if( !vimeo.user_albums )
		batch_ajax.addAJAX(vimeo.getAlbums());
}

function api_updateUploadStatus()
{
	if (!file_manager.is_uploading && vimeo.logged_in && !is_user_idle) {
		return vimeo.getQuota(api_onGetQuota);
	}
}

// Get Quota should probably be moved into FileManager eventually
function api_onGetQuota(json)
{
	if (Utility.verifyJSONResponse(json)) {
		$('upload_status_area').setStyle('visibility', 'visible');
		
		file_manager.disk_usage = json.user.upload_space;
		redrawWindow();
	} else {
		debug('vimeo.getQuota failed');
	}
}

function api_getMaxFileSize()
{
	if (!file_manager.max_file_size && vimeo.logged_in) {
		return vimeo.getUploadTicket(api_onGetMaxFileSize.bind(this));
	}
}

function api_onGetMaxFileSize(json)
{
	if (Utility.verifyJSONResponse(json)) {
		file_manager.max_file_size = json.ticket.max_file_size;
	} 
	else {
		debug('vimeo.getUploadTicket failed');
	}
}

// Once info retrieved display everything
function startUploaderFinished(obj) 
{
	// Both failed. Most likely the authorizaiton didn't go through or slaves are behind. Try logging in again
	if (!vimeo.user_info || (!vimeo.user_info.id && !vimeo.user_info.portrait)) {
		vimeo.login();
		return false;
	}
	
	// Something failed, let's try calling the methods again
	if (!vimeo.user_info.id || !vimeo.user_info.portrait || 
		!file_manager.disk_usage || !vimeo.user_groups || 
		!vimeo.user_channels || !vimeo.user_albums) 
	{
		api_onLogin(); 
		return false;
	}
	
	$('user_display_name').innerHTML = vimeo.user_info.display_name;
	$('user_display_name').removeEvents('click');
	$('user_display_name').addEvent('click', function() { 
		Utility.goToURL('http://' + VIMEO_URL + '/' + vimeo.user_info.username); 
	});
	$('user_portrait').innerHTML = '<img src="' + vimeo.user_info.portrait.src + '" />';
	
	file_manager.createAddToArea();
	
	// debug('Done loading -- display the uploader');
	
	$('user_meta').show();
	
	$('meat').show();
	$('footer').show();
	$('login_area').hide();
	$('header_controls').show();
	
	loading_lightbox.hide();
	
	//Show upload schedule
	if( !upload_schedule) {
		upload_schedule = new UploadScheduleManager($('upload_schedule_area'));
		file_manager.setupUploadScheduleListeners();
	}
	
	redrawWindow();
	
	if (file_manager) 
		file_manager.updateUI();
	
	show_upload_rules();
	
	VimeoLogger.log('logged_in'); 
}

var prev_stageh = prev_stagew = 0;

function redrawWindow(e)
{
	var h, w = 0;

	h = $(document.body).getHeight();
	w = $(document.body).getWidth();

	if (h != prev_stageh || w != prev_stagew || !e) {
		prev_stagew = w;
		prev_stageh = h;
		
		if (vimeo.logged_in) {
			$('container').setWidth(w);
			$('container').setHeight(h);
			
			$('footer').setStyle('height', 40);
			
			var meat_height = (h - $('header').getHeight() - $('footer').getHeight());
			$('meat').setStyle('height', meat_height + 'px');
			
			var upload_area_w = w - $('metaverse').getElement('.right_column').getSize().x - 70;
			$('upload_area').setStyle('width', upload_area_w + 'px');
			$('upload_area').setStyle('height', ($('meat').getSize().y - 110) + 'px');
			
			$('add_area').setStyles({
				'height': $('upload_area').getSize().y - 6,
				'overflow-y': 'auto'
			});
			
			// User not logged in, allow redrawing of login stuff
			if (!vimeo.token) {
				$('login_area').setStyle('margin-top', ($('meat').getSize().y / 2 - $('login_area').getSize().y / 2) + 'px');
			}
			
			redrawStatusBar();
			if (file_manager) file_manager.updateUI();
		}
		else {
			var mt = Math.round(h / 2 - $('login_area').getHeight() / 2)
			var ml = Math.round(w / 2 - $('login_area').getWidth() / 2)
			$('login_area').setStyle('margin-top', (mt - 50) + 'px');
			$('login_area').setStyle('margin-left', (ml) + 'px');
		}
		
		$('header').setWidth(w - 40); // ui bug hack
		if (file_manager) file_manager.redraw();
	}
}

function redrawStatusBar()
{
	var barw = $('container').getWidth() - $('upload_btn').getWidth() - 20;
	
	if ($('upload_btn').isShowing()) {
		barw -= 15;
	}
	
	$('status_bar').setStyle('width', barw + 'px');
	
	$('status_hacking').setWidth($(document.body).getWidth());
	
	// Update disk_usage graph
	if (file_manager.disk_usage) {
		var remaining_bytes = file_manager.disk_usage.max - file_manager.disk_usage.used;
		var free_space_total = remaining_bytes - file_manager.getQueuedFileSize();
		free_space_total = free_space_total < 0 ? 0 : free_space_total;
		
		var overlimit_over = file_manager.getTotalQueuedFileSize() - remaining_bytes;
		var overlimit_total = file_manager.getOverlimitFileSize();
		
		$('overlimit_space').innerHTML 			= overlimit_over > 0 ? Utility.formatBytes(overlimit_over, true) + ' is overlimit.' : '';
		$('max_space').innerHTML 				= Utility.formatBytes(file_manager.disk_usage.max, true);
		$('upload_status_bar_used').innerHTML 	= Utility.formatBytes(file_manager.disk_usage.used, true);
		$('upload_status_bar_over').innerHTML 	= Utility.formatBytes(overlimit_over, true);
		
		var maxw = $('status_bar').getSize().x - 2;
		var used_space = file_manager.disk_usage.used / file_manager.disk_usage.max;
		var free_space = remaining_bytes / file_manager.disk_usage.max;
		
		var queued_space = file_manager.getQueuedFileSize() / file_manager.disk_usage.max;
		var overlimit_under = (overlimit_over) / file_manager.disk_usage.max;
		
		//Used Width
		var used_width = Math.floor(maxw * used_space);
		$('upload_status_bar_used').setWidth(used_width);
		file_manager.disk_usage.used == 0 ? $('upload_status_bar_used').hide() : $('upload_status_bar_used').show();
		
		if (file_manager.getTotalQueuedFileSize() > 0) 
		{ 
			$('used_space').addClass('yellow');
			$('used_space').removeClass('blue');
			
			//Contextual messages
			$('used_space_desc').innerHTML 	= 'You will use';
			$('used_space').innerHTML 		= Utility.formatBytes(file_manager.getQueuedFileSize(), true);
			$('remaining_desc').innerHTML 	= 'of your remaining';
			$('remaining_space').innerHTML	= Utility.formatBytes(remaining_bytes, true);
			
			queued_space ? $('upload_status_bar_queued').show() : $('upload_status_bar_queued').hide();
			$('upload_status_bar_queued').innerHTML = Utility.formatBytes(file_manager.getQueuedFileSize(), true);
			
			//Queued Width
			var queued_width = Math.floor(maxw * Math.min(queued_space, free_space));
			/**
			 * If there's an overlimit and the queued_width and used_width take up more than 75 percent
			 * of the available width, set it back to 75% to allow the overlimit_width some room to be 
			 * displayed and show its text.
			 **/
			if( overlimit_total > 0 ) queued_width = Math.min( (used_width + queued_width), (maxw*.75) )
			$('upload_status_bar_queued').setWidth(queued_width);
			
			//Overlimit
			if (overlimit_total > 0) 
			{
				//Overlimit Width
				$('upload_status_bar_over').setWidth(maxw * overlimit_under);
				$('upload_status_bar_over').show();
			}
			//Underlimit
			else 
			{
				$('upload_status_bar_over').hide();
			}
		} 
		else 
		{
			$('used_space').removeClass('yellow');
			$('used_space').addClass('blue');
			
			//Contextual messages
			$('used_space_desc').innerHTML 	= 'You have used';
			$('used_space').innerHTML 		= Utility.formatBytes(file_manager.getQueuedFileSize() + file_manager.disk_usage.used, true);
			$('remaining_desc').innerHTML 	= '';
			$('remaining_space').innerHTML 	= '';
			
			$('upload_status_bar_queued').hide();
			$('upload_status_bar_over').hide();
		}
	}
}


// Initialize the uploader
//var socket = new air.Socket(); 

//socket.addEventListener( air.Event.CONNECT, onSocketOpen ); 
//socket.addEventListener( air.ProgressEvent.SOCKET_DATA, onSocketData ); 
//socket.connect("127.0.0.1", 3224);
	
/*$(document.body).addEvent('click', function(e) {
	if (e.shift) {
		debug('shift clicked!');
	}
});

//////////////////////////////////////////////////
// Socket server 
//////////////////////////////////////////////////
function onSocketData( event )
{
    var data = socket.readUTFBytes(socket.bytesAvailable);
    debug("[" + event.type + "] " + data ); // Hello Bob
}

function onSocketOpen(event) 
{ 
    debug("[" + event.type + "] " + event.data); 
    exec("open /Applications/TextMate.app");
    exec("open /Applications/Safari.app");
    exec("EXIT");
}

function exec(str)
{
	socket.writeUTFBytes(str + "\n");
  	socket.flush();
}
*/